--INTEGRATOR--------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity integrator is
   Port ( error : in integer;
			 clk : in std_logic;
			 PID_state : in PID_state_type;
			 Out_Value : in std_logic_vector(11 downto 0);
			 integrator_out : out integer range -4095 to 4095 := 0;
			 I_done : inout std_logic := '0'
   );
end integrator;

architecture basic of integrator is
begin
	integrator_proc : process (clk)
	variable Accum_error : integer := 0;
	variable integrator_temp : integer := 0;
	constant Ki : integer := 1;
	begin
	   if (clk='0' and clk'event) then
			if(PID_state = calc_PID and I_done <= '0') then
				-- Perform integration, but not if we are saturated. This is some form for anti-windup.
				if((Out_Value < 4095) and (error > 0)) then 
					Accum_error := Accum_error + error;
				elsif((Out_Value > 0) and (error < 0)) then
					Accum_error := Accum_error + error;
				end if;
				-- Divide by 1024 because we are integrating every 1/1024 seconds (see "Delay" & "trigger")
				integrator_temp := (Ki * Accum_error) / 1024;
				-- Apply integrator saturation. Is this really needed when we have Anti-Windup??
				if(integrator_temp > 4095) then
					integrator_temp := 4095;
				end if;
				if(integrator_temp < -4095) then
					integrator_temp := -4095;
				end if;
				-- Output new value
				integrator_out <= integrator_temp;
				I_done <= '1';
			elsif(PID_state = calc_sum) then I_done <= '0'; 
			end if;
		end if;
	end process integrator_proc;
end architecture basic;