----------------------------------------------------------------------------------
-- Company: Siemens Oil & Gas
-- Engineer: Erik Nss
-- 
-- Create Date:    23:06:03 01/31/2009 
-- Design Name:    PID Regulator
-- Module Name:    PID_Reg - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Masters assignment,  under supervision by Karstein Kristiansen (Siemens) &
-- 				 Sverre Hendseth (NTNU - Teknisk Kybernetikk)
--
-- Dependencies: ChipScope must probably be installed to compile properly. This can be downloaded
--					  for free from Xilinx's webpages.
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--FULL REGULATOR------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity PID_Regulator is
   Port ( clk : in std_logic;
			 --SPI
			 SPI_MISO : in std_logic;
			 SPI_MOSI : out std_logic;
			 SPI_SCK : inout std_logic;
			 --UART
			 UART_RXD : in std_logic;
			 UART_TXD : out std_logic;
			 --ADC and Ampifier
			 AD_CONV : out std_logic;
			 AMP_CS : out std_logic;
			 AMP_SHDN : out std_logic;
			 --DAC
			 DAC_CS : out std_logic;
			 DAC_CLR : out std_logic;
			 --LCD-interface
			 SF_D_11,SF_D_10,SF_D_9,SF_D_8 : out std_logic;
			 LCD_E,LCD_RS,LCD_RW : out std_logic;
			 --LEDS
			 LED_0, LED_1, LED_2, LED_3, LED_4, LED_5, LED_6, LED_7 : out std_logic;
			 --VGA
			 r, g, b, hsync, vsync : out std_logic;
			 --BUTTONS
			 ROT_CENTER : in std_logic;
			 ROT_A : in std_logic := '0';
			 ROT_B : in std_logic := '0';
			 BTN_NORTH : in std_logic := '0';
			 BTN_WEST : in std_logic := '0';
			 BTN_SOUTH : in std_logic := '0';
			 BTN_EAST : in std_logic := '0';
			 --SPI-Devices to be disabled
			 SPI_SS_B : out std_logic;
			 SF_CE0 : out std_logic;
			 FPGA_INIT_B : out std_logic
   );
end PID_Regulator;

architecture struct of PID_Regulator is
	--Internal signals
	--VGA signals
	--signal VGA_CLK : std_logic;									--This is the 25 MHz VGA clock.
	--signal red, green, blue : std_logic;
	--signal row : std_logic_vector(8 downto 0);
	--signal column : std_logic_vector(9 downto 0);
	--RAM signals
	--signal RAM_dto: std_logic_VECTOR(31 downto 0);
	--signal RAM_addr: std_logic_VECTOR(11 downto 0);
	--signal RAM_we: std_logic_VECTOR(0 downto 0);
	--signal RAM_dfrom: std_logic_VECTOR(31 downto 0);
	--UART
	signal UART_CLK : std_logic;	
	signal UART_DATA : UART_queue;
	--MPC regulator
	signal P_Value_mod : std_logic_vector(11 downto 0);	--Model state 1
	--Ahat signals
	signal Ahat_dto: std_logic_VECTOR(31 downto 0);
	signal Ahat_addr: std_logic_VECTOR(5 downto 0);			--(5:1) Row (0:0) Col
	signal Ahat_we: std_logic_VECTOR(0 downto 0);
	signal Ahat_dfrom: std_logic_VECTOR(31 downto 0);
	--Bhat signals
	signal Bhat_dto: std_logic_VECTOR(31 downto 0);
	signal Bhat_addr: std_logic_VECTOR(8 downto 0);			--(8:4) Row (3:0) Col
	signal Bhat_we: std_logic_VECTOR(0 downto 0);
	signal Bhat_dfrom: std_logic_VECTOR(31 downto 0);
	--Xdev signals
	signal Xdev_dto: std_logic_VECTOR(31 downto 0);
	signal Xdev_addr: std_logic_VECTOR(4 downto 0);
	signal Xdev_we: std_logic_VECTOR(0 downto 0);
	signal Xdev_dfrom: std_logic_VECTOR(31 downto 0);
	--J1 signals
	signal J1_dto: std_logic_VECTOR(31 downto 0);
	signal J1_addr: std_logic_VECTOR(9 downto 0);			--(9:4) Row (3:0) Col
	signal J1_we: std_logic_VECTOR(0 downto 0);
	signal J1_dfrom: std_logic_VECTOR(31 downto 0);
	--G1 signals
	signal G1_dto: std_logic_VECTOR(31 downto 0);
	signal G1_addr: std_logic_VECTOR(5 downto 0);
	signal G1_we: std_logic_VECTOR(0 downto 0);
	signal G1_dfrom: std_logic_VECTOR(31 downto 0);
	--Q signals
	signal Q_dto: std_logic_VECTOR(31 downto 0);
	signal Q_addr: std_logic_VECTOR(7 downto 0);				--(7:4) Row (3:0) Col
	signal Q_we: std_logic_VECTOR(0 downto 0);
	signal Q_dfrom: std_logic_VECTOR(31 downto 0);
	--Temp1 signals
	signal Temp1_dto: std_logic_VECTOR(31 downto 0);
	signal Temp1_addr: std_logic_VECTOR(8 downto 0);		--(8:4) Row (3:0) Col
	signal Temp1_we: std_logic_VECTOR(0 downto 0);
	signal Temp1_dfrom: std_logic_VECTOR(31 downto 0);
	--C signals
	signal C_dto: std_logic_VECTOR(31 downto 0);
	signal C_addr: std_logic_VECTOR(3 downto 0);				--(3:0)
	signal C_we: std_logic_VECTOR(0 downto 0);
	signal C_dfrom: std_logic_VECTOR(31 downto 0);
	--Z signals
	signal Z_dto: std_logic_VECTOR(31 downto 0);
	signal Z_addr: std_logic_VECTOR(3 downto 0);			--(3:0)
	signal Z_we: std_logic_VECTOR(0 downto 0);
	signal Z_dfrom: std_logic_VECTOR(31 downto 0);
	--T signals
	signal T_dto: std_logic_VECTOR(31 downto 0);
	signal T_addr: std_logic_VECTOR(5 downto 0);			--(5:0)
	signal T_we: std_logic_VECTOR(0 downto 0);
	signal T_dfrom: std_logic_VECTOR(31 downto 0);
	--L signals
	signal L_dto: std_logic_VECTOR(31 downto 0);
	signal L_addr: std_logic_VECTOR(5 downto 0);			--(5:0)
	signal L_we: std_logic_VECTOR(0 downto 0);
	signal L_dfrom: std_logic_VECTOR(31 downto 0);
	--G2 signals
	signal G2_dto: std_logic_VECTOR(31 downto 0);
	signal G2_addr: std_logic_VECTOR(5 downto 0);			--(5:0)
	signal G2_we: std_logic_VECTOR(0 downto 0);
	signal G2_dfrom: std_logic_VECTOR(31 downto 0);
	--R1 signals
	signal R1_dto: std_logic_VECTOR(31 downto 0);
	signal R1_addr: std_logic_VECTOR(3 downto 0);			--(3:0)
	signal R1_we: std_logic_VECTOR(0 downto 0);
	signal R1_dfrom: std_logic_VECTOR(31 downto 0);
	--R2 signals
	signal R2_dto: std_logic_VECTOR(31 downto 0);
	signal R2_addr: std_logic_VECTOR(5 downto 0);			--(5:0)
	signal R2_we: std_logic_VECTOR(0 downto 0);
	signal R2_dfrom: std_logic_VECTOR(31 downto 0);
	--Temp2 signals
	signal Temp2_dto: std_logic_VECTOR(31 downto 0);
	signal Temp2_addr: std_logic_VECTOR(9 downto 0);			--(9:6) Row (5:0) Col
	signal Temp2_we: std_logic_VECTOR(0 downto 0);
	signal Temp2_dfrom: std_logic_VECTOR(31 downto 0);
	--Temp3 signals
	signal Temp3_dto: std_logic_VECTOR(31 downto 0);
	signal Temp3_addr: std_logic_VECTOR(7 downto 0);			--(7:4) Row (3:0) Col
	signal Temp3_we: std_logic_VECTOR(0 downto 0);
	signal Temp3_dfrom: std_logic_VECTOR(31 downto 0);
	--Temp4 signals
	signal Temp4_dto: std_logic_VECTOR(31 downto 0);
	signal Temp4_addr: std_logic_VECTOR(7 downto 0);			--(7:4) Row (3:0) Col
	signal Temp4_we: std_logic_VECTOR(0 downto 0);
	signal Temp4_dfrom: std_logic_VECTOR(31 downto 0);
	--R signals
	signal R_dto: std_logic_VECTOR(31 downto 0);
	signal R_addr: std_logic_VECTOR(3 downto 0);					--(3:0)
	signal R_we: std_logic_VECTOR(0 downto 0);
	signal R_dfrom: std_logic_VECTOR(31 downto 0);
	
	--Arithmetics
	--Used for MPC
	signal float_a : std_logic_VECTOR(31 downto 0);
	signal float_b : std_logic_VECTOR(31 downto 0);
	signal float_op : std_logic_VECTOR(5 downto 0);
	signal float_add_res : std_logic_VECTOR(31 downto 0);
	signal float_comp_res : std_logic_VECTOR(0 downto 0);
	signal float_mul_res : std_logic_VECTOR(31 downto 0);
	signal float_div_res : std_logic_VECTOR(31 downto 0);
   signal std_to_float : std_logic_VECTOR(15 downto 0);
	signal float_gen : std_logic_VECTOR(31 downto 0);
	signal float_to_std : std_logic_VECTOR(31 downto 0);
	signal std_gen : std_logic_VECTOR(15 downto 0);
	
	--other
	signal reset : std_logic;										--Resets the system. Using BTN_EAST.
	signal P_Value : std_logic_vector(11 downto 0);			--Process Value read from ADC: 0 to 4095
	signal Ref : std_logic_vector(7 downto 0);				--Reference
	signal debug : std_logic_vector(31 downto 0); 			--Used for debugging.
	signal Out_Value : std_logic_vector(11 downto 0);		--Value to sent to DAC/process ("paadrag")
	signal SYS_state : SYS_state_type; 							--Global statemachine
	signal SPI_done, MPC_done : std_logic;						--Signals for trigging the statemachine
	signal Xref : array1;
begin
	--Clock generator
	b_clk_gen : entity work.clk_gen(basic)
		port map (clk, SPI_SCK, UART_CLK);
	--UART
	b_UART : entity work.uart(basic)
		port map (UART_CLK, UART_RXD, UART_TXD, UART_DATA, Xref, debug);
	--Other
	--b_ref_gen : entity work.ref_gen(basic)
	   --port map (clk, ROT_A, ROT_B, Ref);
	b_spi : entity work.spi_interface(basic)
	   port map (SPI_SCK, SPI_MISO, SPI_MOSI, AMP_CS, AMP_SHDN, SYS_state,
				    AD_CONV, P_Value, DAC_CS, DAC_CLR, Out_Value, SPI_done);
	--Regulator
	b_MPC : entity work.MPC(basic)
		port map (clk, Xref, P_value, Out_Value, debug, SYS_state, MPC_done,
					 float_a, float_b, float_op, float_add_res, float_comp_res, float_mul_res, float_div_res, 
					 std_to_float, float_gen, float_to_std, std_gen,
					 Ahat_dto, Ahat_addr, Ahat_we, Ahat_dfrom,
					 Bhat_dto, Bhat_addr, Bhat_we, Bhat_dfrom,
					 Xdev_dto, Xdev_addr, Xdev_we, Xdev_dfrom,
					 J1_dto, J1_addr, J1_we, J1_dfrom,
					 G1_dto, G1_addr, G1_we, G1_dfrom,
					 Q_dto, Q_addr, Q_we, Q_dfrom,
					 Temp1_dto, Temp1_addr, Temp1_we, Temp1_dfrom,
					 C_dto, C_addr, C_we, C_dfrom,
					 Z_dto, Z_addr, Z_we, Z_dfrom,
					 T_dto, T_addr, T_we, T_dfrom,
					 L_dto, L_addr, L_we, L_dfrom,
					 G2_dto, G2_addr, G2_we, G2_dfrom,
					 R1_dto, R1_addr, R1_we, R1_dfrom,
					 R2_dto, R2_addr, R2_we, R2_dfrom,
					 Temp2_dto, Temp2_addr, Temp2_we, Temp2_dfrom,
					 Temp3_dto, Temp3_addr, Temp3_we, Temp3_dfrom,
					 Temp4_dto, Temp4_addr, Temp4_we, Temp4_dfrom,
					 R_dto, R_addr, R_we, R_dfrom
					 );

	--LED and LCD controller
	--b_LED_CTRL : entity work.leds(basic)
		--port map (clk, LED_0, LED_1, LED_2, LED_3, LED_4, LED_5, LED_6, LED_7, Ref);
	--b_LCD_CTRL : entity work.lcd(basic)
		--port map (clk, reset, SF_D_11, SF_D_10, SF_D_9, SF_D_8, LCD_E, LCD_RS, LCD_RW, P_Value, Out_Value);
	--VGA
	--b_vga : entity work.vga(basic)
	   --port map (VGA_CLK, red, green, blue, r, g, b, hsync, vsync, row, column);
	--b_screen_gen : entity work.screen_gen(basic)
		--port map (VGA_CLK, row, column, red, green, blue, Ref, mode, P_Value, P_Value_mod, Out_Value, RAM_dto, RAM_addr, RAM_we, RAM_dfrom, BTN_NORTH, BTN_SOUTH, ROT_CENTER, UART_DATA, Kp, Ti, Td);
	--Other
	b_SYS_SM : entity work.SYS_SM(basic)
		port map (clk, SYS_state, SPI_done, MPC_done);
	b_GROUND : entity work.ground(basic)
		port map (clk,b,g,r,hsync,vsync,
					 LED_0,LED_1,LED_2,LED_3,LED_4,LED_5,LED_6,LED_7,
					 SF_D_8,SF_D_9,SF_D_10,SF_D_11,LCD_E,LCD_RS,LCD_RW,
					 SPI_SS_B, SF_CE0, FPGA_INIT_B);
		
	------------------- COREGEN BLOCKS ------------------------		
	------------------- Memory storing our matrices
	b_Ahat : entity work.ahat(Ahat_a)
		port map (clk, Ahat_dto, Ahat_addr, Ahat_we, Ahat_dfrom);
	b_Bhat : entity work.bhat(Bhat_a)
		port map (clk, Bhat_dto, Bhat_addr, Bhat_we, Bhat_dfrom);
	b_Xdev : entity work.xdev(Xdev_a)
		port map (clk, Xdev_dto, Xdev_addr, Xdev_we, Xdev_dfrom);
	b_J1 : entity work.J1(J1_a)
		port map (clk, J1_dto, J1_addr, J1_we, J1_dfrom);
	b_G1 : entity work.G1(G1_a)
		port map (clk, G1_dto, G1_addr, G1_we, G1_dfrom);
	b_Q : entity work.Q(Q_a)
		port map (clk, Q_dto, Q_addr, Q_we, Q_dfrom);
	b_Temp1 : entity work.Temp1(Temp1_a)
		port map (clk, Temp1_dto, Temp1_addr, Temp1_we, Temp1_dfrom);
	b_C : entity work.C(C_a)
		port map (clk, C_dto, C_addr, C_we, C_dfrom);
	b_Z : entity work.Z(Z_a)
		port map (clk, Z_dto, Z_addr, Z_we, Z_dfrom);	
	b_T : entity work.T(T_a)
		port map (clk, T_dto, T_addr, T_we, T_dfrom);
	b_L : entity work.L(L_a)
		port map (clk, L_dto, L_addr, L_we, L_dfrom);
	b_G2 : entity work.G2(G2_a)
		port map (clk, G2_dto, G2_addr, G2_we, G2_dfrom);
	b_R1 : entity work.R1(R1_a)
		port map (clk, R1_dto, R1_addr, R1_we, R1_dfrom);	
	b_R2 : entity work.R2(R2_a)
		port map (clk, R2_dto, R2_addr, R2_we, R2_dfrom);	
	b_Temp2 : entity work.Temp2(Temp2_a)
		port map (clk, Temp2_dto, Temp2_addr, Temp2_we, Temp2_dfrom);
	b_Temp3 : entity work.Temp3(Temp3_a)
		port map (clk, Temp3_dto, Temp3_addr, Temp3_we, Temp3_dfrom);
	b_Temp4 : entity work.Temp4(Temp4_a)
		port map (clk, Temp4_dto, Temp4_addr, Temp4_we, Temp4_dfrom);
	b_R : entity work.R(R_a)
		port map (clk, R_dto, R_addr, R_we, R_dfrom);

	--Mathematical functions (float operations)
	b_float_add : entity work.float_add(float_add_a)
		port map (float_a, float_b, float_op, clk, float_add_res);
	b_float_comp : entity work.float_comp(float_comp_a)
		port map (float_a, float_b, clk, float_comp_res);
	b_float_mul : entity work.float_mul(float_mul_a)
		port map (float_a, float_b, clk, float_mul_res);
	b_float_div : entity work.float_div(float_div_a)
		port map (float_a, float_b, clk, float_div_res);
	b_float_gen : entity work.float_gen(basic)							-- std_logic_vec to float
		port map (clk, std_to_float, float_gen);
	b_std_gen : entity work.std_gen(basic)									-- float to std_logic_vec
		port map (clk, float_to_std, std_gen);
			
end architecture struct;