--SPI INTERFACE-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity spi_interface is
   Port ( SCK : in std_logic;
			 MISO : in std_logic;
			 MOSI : out std_logic := '0';
			 AMP_CS : out std_logic := '1';
			 AMP_SHDN : out std_logic := '0';
			 SYS_state : in SYS_state_type;
			 AD_CONV : out std_logic := '0';
			 P_Value : out std_logic_vector(11 downto 0) := "000000000000";
			 DAC_CS : out std_logic := '1';
			 DAC_CLR : out std_logic := '1';
			 Out_Value : in std_logic_vector(11 downto 0);
			 SPI_done : out std_logic := '0'
   );
end spi_interface;

architecture basic of spi_interface is
type SPI_state_type is (init, idle, G_start, G_bit0, G_bit1, G_bit2, G_bit3, G_bit4, G_bit5, G_bit6, G_bit7, G_stop,
							ADC_start, ADC_start2, ADC_start3, DELAY1, DELAY2, DELAY3, DELAY4, DELAY5, DELAY6, DELAY7, 
							A1_b0, A1_b1, A1_b2, A1_b3, A1_b4, A1_b5, A1_b6, A1_b7, A1_b8, A1_b9, A1_b10, A1_b11, A1_b12, A1_b13,
							A2_b0, A2_b1, A2_b2, A2_b3, A2_b4, A2_b5, A2_b6, A2_b7, A2_b8, A2_b9, A2_b10, A2_b11, A2_b12, A2_b13,
							DAC_START, DAC_CMD_0, DAC_CMD_1, DAC_CMD_2, DAC_CMD_3, DAC_ADR_0, DAC_ADR_1, DAC_ADR_2, DAC_ADR_3,
							DAC_0, DAC_1, DAC_2, DAC_3, DAC_4, DAC_5, DAC_6, DAC_7, DAC_8, DAC_9, DAC_10, DAC_11,
							finished
							);
signal SPI_state : SPI_state_type := init; -- transmitter's SPI_state
begin
	spi_proc : process (SCK) is
	constant A_Gain : std_logic_vector(3 downto 0) := "0001"; --"0001" = -1
	constant B_Gain : std_logic_vector(3 downto 0) := "0001"; --"0001" = -1
	variable ADC_IN1 : std_logic_vector(13 downto 0) := "00000000000000";
	variable ADC_IN2 : std_logic_vector(13 downto 0) := "00000000000000";
	variable P_Value_Buf : std_logic_vector(13 downto 0);
	begin
	   if (SCK'event and SCK='0') then
			case SPI_state is
				when init => 
					SPI_state <= G_start;
				when idle => 
					SPI_done <= '0';
					if(SYS_state = Run_SPI) then 
						SPI_state <= ADC_start;
					end if;
				--SET GAIN ON AMPLIFIER
				when G_start => SPI_state <= G_bit0; AMP_CS <= '0'; 
				when G_bit0 => SPI_state <= G_bit1; MOSI <= B_Gain(3);
				when G_bit1 => SPI_state <= G_bit2; MOSI <= B_Gain(2);
				when G_bit2 => SPI_state <= G_bit3; MOSI <= B_Gain(1);
				when G_bit3 => SPI_state <= G_bit4; MOSI <= B_Gain(0);
				when G_bit4 => SPI_state <= G_bit5; MOSI <= A_Gain(3);
				when G_bit5 => SPI_state <= G_bit6; MOSI <= A_Gain(2);
				when G_bit6 => SPI_state <= G_bit7; MOSI <= A_Gain(1);
				when G_bit7 => SPI_state <= G_stop; MOSI <= A_Gain(0);
				when G_stop => SPI_state <= idle; AMP_CS <= '1'; MOSI <= '0';
				--READ INPUT FROM ADC
				when ADC_start => SPI_state <= ADC_START2; AD_CONV <= '1';
				when ADC_start2 => SPI_state <= ADC_start3; AD_CONV <= '0';
				when ADC_start3 => SPI_state <= A1_b0; 
				when A1_b0 => SPI_state <= A1_b1; ADC_IN1(13) := MISO;
				when A1_b1 => SPI_state <= A1_b2; ADC_IN1(12) := MISO;
				when A1_b2 => SPI_state <= A1_b3; ADC_IN1(11) := MISO;
				when A1_b3 => SPI_state <= A1_b4; ADC_IN1(10) := MISO;
				when A1_b4 => SPI_state <= A1_b5; ADC_IN1(9) := MISO;
				when A1_b5 => SPI_state <= A1_b6; ADC_IN1(8) := MISO;
				when A1_b6 => SPI_state <= A1_b7; ADC_IN1(7) := MISO;
				when A1_b7 => SPI_state <= A1_b8; ADC_IN1(6) := MISO;
				when A1_b8 => SPI_state <= A1_b9; ADC_IN1(5) := MISO;
				when A1_b9 => SPI_state <= A1_b10; ADC_IN1(4) := MISO;
				when A1_b10 => SPI_state <= A1_b11; ADC_IN1(3) := MISO;
				when A1_b11 => SPI_state <= A1_b12; ADC_IN1(2) := MISO;
				when A1_b12 => SPI_state <= A1_b13; ADC_IN1(1) := MISO;
				when A1_b13 => SPI_state <= DELAY1; ADC_IN1(0) := MISO;
				when DELAY1 => SPI_state <= DELAY2;
				when DELAY2 => SPI_state <= A2_b0;
				when A2_b0 => SPI_state <= A2_b1; ADC_IN2(13) := MISO;
				when A2_b1 => SPI_state <= A2_b2; ADC_IN2(12) := MISO;
				when A2_b2 => SPI_state <= A2_b3; ADC_IN2(11) := MISO;
				when A2_b3 => SPI_state <= A2_b4; ADC_IN2(10) := MISO;
				when A2_b4 => SPI_state <= A2_b5; ADC_IN2(9) := MISO;
				when A2_b5 => SPI_state <= A2_b6; ADC_IN2(8) := MISO;
				when A2_b6 => SPI_state <= A2_b7; ADC_IN2(7) := MISO;
				when A2_b7 => SPI_state <= A2_b8; ADC_IN2(6) := MISO;
				when A2_b8 => SPI_state <= A2_b9; ADC_IN2(5) := MISO;
				when A2_b9 => SPI_state <= A2_b10; ADC_IN2(4) := MISO;
				when A2_b10 => SPI_state <= A2_b11; ADC_IN2(3) := MISO;
				when A2_b11 => SPI_state <= A2_b12; ADC_IN2(2) := MISO;
				when A2_b12 => SPI_state <= A2_b13; ADC_IN2(1) := MISO;
				when A2_b13 => SPI_state <= DAC_START; ADC_IN2(0) := MISO;
				--WRITE OUTPUT TO DAC
				when DAC_START => SPI_state <= DAC_CMD_0; DAC_CS <= '0'; 
				when DAC_CMD_0 => SPI_state <= DAC_CMD_1; MOSI <= '0'; --CMD "0011"
				when DAC_CMD_1 => SPI_state <= DAC_CMD_2; MOSI <= '0';
				when DAC_CMD_2 => SPI_state <= DAC_CMD_3; MOSI <= '1';
				when DAC_CMD_3 => SPI_state <= DAC_ADR_0; MOSI <= '1'; 
				when DAC_ADR_0 => SPI_state <= DAC_ADR_1; MOSI <= '0'; --ADR "0000" = DAC0
				when DAC_ADR_1 => SPI_state <= DAC_ADR_2; MOSI <= '0';
				when DAC_ADR_2 => SPI_state <= DAC_ADR_3; MOSI <= '0';
				when DAC_ADR_3 => SPI_state <= DAC_0; MOSI <= '0';
				when DAC_0 => SPI_state <= DAC_1; MOSI <= Out_Value(11); 
				when DAC_1 => SPI_state <= DAC_2; MOSI <= Out_Value(10);
				when DAC_2 => SPI_state <= DAC_3; MOSI <= Out_Value(9);
				when DAC_3 => SPI_state <= DAC_4; MOSI <= Out_Value(8);
				when DAC_4 => SPI_state <= DAC_5; MOSI <= Out_Value(7);
				when DAC_5 => SPI_state <= DAC_6; MOSI <= Out_Value(6);
				when DAC_6 => SPI_state <= DAC_7; MOSI <= Out_Value(5);
				when DAC_7 => SPI_state <= DAC_8; MOSI <= Out_Value(4);
				when DAC_8 => SPI_state <= DAC_9; MOSI <= Out_Value(3);
				when DAC_9 => SPI_state <= DAC_10; MOSI <= Out_Value(2);
				when DAC_10 => SPI_state <= DAC_11; MOSI <= Out_Value(1);
				when DAC_11 => SPI_state <= DELAY3; MOSI <= Out_Value(0);
				when DELAY3 => SPI_state <= DELAY4;
				when DELAY4 => SPI_state <= DELAY5;
				when DELAY5 => SPI_state <= DELAY6;
				when DELAY6 => SPI_state <= DELAY7;
				when DELAY7 => SPI_state <= FINISHED; DAC_CS <= '1';
				when FINISHED => SPI_state <= idle;
									  if(ADC_IN2(13) = '1') then
											P_Value_Buf := "11111111111111" - ADC_IN2(12 downto 0);
								     elsif(ADC_IN2(13) = '0') then
											P_Value_Buf := "01111111111111" - ADC_IN2(12 downto 0);	-- 0 - 16383
									  end if;
									  P_Value(11 downto 0) <= P_Value_Buf(13 downto 2); --P_Value is now 0 - 4095
									  SPI_done <= '1';
				when others => SPI_state <= idle;
			end case;
		end if;
	end process spi_proc;	
end architecture basic;