 --MPC block-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;
use work.mpc_package.all;

entity MPC is
   Port ( clk 			 : in std_logic;
			 Xref 		 : in array1;
			 P_value		 : in std_logic_vector(11 downto 0);
			 Out_Value	 : out std_logic_vector(11 downto 0);
			 debug		 : out std_logic_vector(31 downto 0) := "00000000000000000000111111111111";
			 SYS_state 	 : in SYS_state_type;
			 MPC_done 	 : out std_logic := '0';
			 float_a 		: out std_logic_vector(31 downto 0);
			 float_b 		: out std_logic_vector(31 downto 0);
			 float_op 		: out std_logic_VECTOR(5 downto 0);
			 float_add_res	: in std_logic_vector(31 downto 0);
			 float_comp_res : in std_logic_vector(0 downto 0);
			 float_mul_res	: in std_logic_vector(31 downto 0);
			 float_div_res	: in std_logic_vector(31 downto 0);
			 std_to_float 	: out std_logic_VECTOR(15 downto 0);
			 float_gen 		: in std_logic_VECTOR(31 downto 0);
			 float_to_std	: out std_logic_vector(31 downto 0);
			 std_gen			: in std_logic_VECTOR(15 downto 0);
			 Ahat_dto		: out std_logic_VECTOR(31 downto 0);
			 Ahat_addr		: out std_logic_VECTOR(5 downto 0); 
			 Ahat_we			: out std_logic_VECTOR(0 downto 0);
			 Ahat_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Bhat_dto		: out std_logic_VECTOR(31 downto 0);
			 Bhat_addr		: out std_logic_VECTOR(8 downto 0); 
			 Bhat_we			: out std_logic_VECTOR(0 downto 0);
			 Bhat_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Xdev_dto		: out std_logic_VECTOR(31 downto 0);
			 Xdev_addr		: out std_logic_VECTOR(4 downto 0); 
			 Xdev_we			: out std_logic_VECTOR(0 downto 0);
			 Xdev_dfrom		: in std_logic_VECTOR(31 downto 0);
			 J1_dto			: out std_logic_VECTOR(31 downto 0);
			 J1_addr			: out std_logic_VECTOR(9 downto 0); 
			 J1_we			: out std_logic_VECTOR(0 downto 0);
			 J1_dfrom		: in std_logic_VECTOR(31 downto 0);
			 G1_dto			: out std_logic_VECTOR(31 downto 0);
			 G1_addr			: out std_logic_VECTOR(5 downto 0); 
			 G1_we			: out std_logic_VECTOR(0 downto 0);
			 G1_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Q_dto			: out std_logic_VECTOR(31 downto 0);
			 Q_addr			: out std_logic_VECTOR(7 downto 0); 
			 Q_we				: out std_logic_VECTOR(0 downto 0);
			 Q_dfrom			: in std_logic_VECTOR(31 downto 0);
			 Temp1_dto		: out std_logic_VECTOR(31 downto 0);
			 Temp1_addr		: out std_logic_VECTOR(8 downto 0); 
			 Temp1_we		: out std_logic_VECTOR(0 downto 0);
			 Temp1_dfrom	: in std_logic_VECTOR(31 downto 0);
			 C_dto			: out std_logic_VECTOR(31 downto 0);
			 C_addr			: out std_logic_VECTOR(3 downto 0); 
			 C_we				: out std_logic_VECTOR(0 downto 0);
			 C_dfrom			: in std_logic_VECTOR(31 downto 0);
			 Z_dto			: out std_logic_VECTOR(31 downto 0);
			 Z_addr			: out std_logic_VECTOR(3 downto 0); 
			 Z_we				: out std_logic_VECTOR(0 downto 0);
			 Z_dfrom			: in std_logic_VECTOR(31 downto 0);
			 T_dto			: out std_logic_VECTOR(31 downto 0);
			 T_addr			: out std_logic_VECTOR(5 downto 0); 
			 T_we				: out std_logic_VECTOR(0 downto 0);
			 T_dfrom			: in std_logic_VECTOR(31 downto 0);
			 L_dto			: out std_logic_VECTOR(31 downto 0);
			 L_addr			: out std_logic_VECTOR(5 downto 0); 
			 L_we				: out std_logic_VECTOR(0 downto 0);
			 L_dfrom			: in std_logic_VECTOR(31 downto 0);
			 G2_dto			: out std_logic_VECTOR(31 downto 0);
			 G2_addr			: out std_logic_VECTOR(5 downto 0); 
			 G2_we			: out std_logic_VECTOR(0 downto 0);
			 G2_dfrom		: in std_logic_VECTOR(31 downto 0);
			 R1_dto			: out std_logic_VECTOR(31 downto 0);
			 R1_addr			: out std_logic_VECTOR(3 downto 0); 
			 R1_we			: out std_logic_VECTOR(0 downto 0);
			 R1_dfrom		: in std_logic_VECTOR(31 downto 0);
			 R2_dto			: out std_logic_VECTOR(31 downto 0);
			 R2_addr			: out std_logic_VECTOR(5 downto 0); 
			 R2_we			: out std_logic_VECTOR(0 downto 0);
			 R2_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Temp2_dto		: out std_logic_VECTOR(31 downto 0);
			 Temp2_addr		: out std_logic_VECTOR(9 downto 0); 
			 Temp2_we		: out std_logic_VECTOR(0 downto 0);
			 Temp2_dfrom	: in std_logic_VECTOR(31 downto 0);
			 Temp3_dto		: out std_logic_VECTOR(31 downto 0);
			 Temp3_addr		: out std_logic_VECTOR(7 downto 0); 
			 Temp3_we		: out std_logic_VECTOR(0 downto 0);
			 Temp3_dfrom	: in std_logic_VECTOR(31 downto 0);
			 Temp4_dto		: out std_logic_VECTOR(31 downto 0);
			 Temp4_addr		: out std_logic_VECTOR(7 downto 0); 
			 Temp4_we		: out std_logic_VECTOR(0 downto 0);
			 Temp4_dfrom	: in std_logic_VECTOR(31 downto 0);
			 R_dto			: out std_logic_VECTOR(31 downto 0);
			 R_addr			: out std_logic_VECTOR(3 downto 0); 
			 R_we				: out std_logic_VECTOR(0 downto 0);
			 R_dfrom			: in std_logic_VECTOR(31 downto 0)
		);
end MPC;

architecture basic of MPC is
type MPC_state_type is (init, get_x0, create_obj, calc_min, idle);
signal MPC_state : MPC_state_type := init; -- MPC-regulator state machine

type init_state_type is (idle, delay, gen_ahat_1, gen_ahat_2, gen_bhat_1, gen_bhat_2, gen_bhat_3);	--MPC init states
signal init_state : init_state_type := gen_ahat_1;
signal init_nextstate : init_state_type := gen_ahat_1;

type MODEL_state_type is (calc_states_1, calc_states_2, calc_states_3, calc_states_4, 
								  calc_states_5, calc_states_6, calc_states_7, calc_states_8, 
								  calc_states_9, calc_states_10, calc_states_11, calc_states_12,
								  calc_states_13, delay);
signal MODEL_state : MODEL_state_type := calc_states_1; -- Statemachine for modelling process
signal MODEL_nextstate : MODEL_state_type := calc_states_1; -- Statemachine for modelling process

type obj_state_type is (delay, gen_xdev_1, gen_xdev_2, gen_J1_1, gen_J1_2, gen_G1_1, gen_G1_2, 
								gen_Q_1, gen_Q_2, gen_Q_3, gen_C_1, range_Q_C, range_J_G, gen_ZLT);
signal obj_state : obj_state_type := gen_xdev_1;
signal obj_nextstate : obj_state_type := gen_xdev_1;

type calc_min_state_type is (delay, gen_output, gen_G2_1, gen_R1_1, gen_R1_2, gen_r2_1, gen_r2_2, gen_D_Z_1, gen_D_Z_2,
									  gen_D_Z_3, gen_INV_1, gen_INV_2, gen_D_Z_4, gen_D_Z_5, clr_Temp2, gen_D_Z_6, gen_Z_1,
									  gen_L_1, gen_L_2, gen_T_1, gen_T_2, gen_my_1, gen_my_2, gen_my_3);
signal calc_min_state : calc_min_state_type := gen_G2_1;
signal calc_min_nextstate : calc_min_state_type := gen_G2_1;

begin
	MPC_proc : process (clk) is
	variable my 			 : std_logic_vector(31 downto 0) := "00111111100000000000000000000000";
	variable n 				 : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	variable usedalpha 	 : std_logic_vector(31 downto 0) := alpha;
	variable tempvar 	    : std_logic_vector(31 downto 0); --Used for d_t & d_l when modifying alpha
	variable prev_Z_dfrom : std_logic_vector(31 downto 0) := "00000000000000000000000000000000"; --Used for output rollback
	--used for "model"
	variable x0 	: array1 	:= ("00000000000000000000000000000000","00000000000000000000000000000000");
	variable x_int : array1  	:= ("00000000000000000000000000000000","00000000000000000000000000000000");
	variable Out_value_float : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	--These are the loop variables
	variable g : integer := 0;
	variable h : integer := 0;
	variable i : integer := 0;
	variable j : integer := 0;
	variable k : integer := 0;
	variable loopnum : integer := 0;	--Used in the matrix inversion
	--These are used for finding the max value in the matrixes, used for changing range to -1 < x < 1
	variable Q_C_max : std_logic_vector(31 downto 0) := "00111111100000000000000000000000";
	variable J_G_max : std_logic_vector(31 downto 0) := "00111111100000000000000000000000";
	variable temp_max : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	--Other stuff
	variable counter	 : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	begin
	   if (clk='1' and clk'event) then
			case MPC_state is
				----------------------- Initiate all matrices needed for the QP-solver ---------------
				when init =>
					case init_state is
						when gen_ahat_1 =>	--Put A on top
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector(i, Ahat_r) & conv_std_logic_vector(j, Ahat_c);
									Ahat_dto <= Ad(i)(j); 
									Ahat_we <= "1";
								when others => g:=0;
							end case;
							init_nextstate <= gen_ahat_1;
							g := g+1;
							if(g=1) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+1;
									if(i=n_x) then
										init_nextstate <= gen_ahat_2;
										g := 0;h := 0;i := 0;j := 0;k := 1;
									end if;
								end if;
							end if;
							init_state <= delay;

						when gen_ahat_2 =>	--Generate A^2, A^3 etc downwards
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector(i+(k-1)*n_x, Ahat_r) & conv_std_logic_vector(h, Ahat_c);
									Ahat_we <= "0";
								when 1 => --We should multiply
									float_a <= Ahat_dfrom;
									float_b <= Ad(h)(j);
								when 2 => --Get value from Ahat where multiplication is "added to"
									Ahat_addr <= conv_std_logic_vector(i+k*n_x, Ahat_r) & conv_std_logic_vector(j, Ahat_c);
									Ahat_we <= "0";
								when 3 => --We should add
									float_a <= float_mul_res;
									float_b <= Ahat_dfrom;
									float_op <= "000000";
								when 4 => --Write new value
									Ahat_addr <= conv_std_logic_vector(i+k*n_x, Ahat_r) & conv_std_logic_vector(j, Ahat_c);
									Ahat_dto <= float_add_res;
									Ahat_we <= "1";
								when others => g := 0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								h := h+1;
								if(h=n_x) then
									h:=0;
									j := j+1;
									if(j=n_x) then
										j:=0;
										i:=i+1;
										if(i=n_x) then
											i:=0;
											k:=k+1;
											if(k=n_pred) then
												k := 0;
												init_nextstate <= gen_bhat_1;
											end if;
										end if;
									end if;
								end if;
							end if;
							init_state <= delay;
								
						when gen_bhat_1 =>	-- But Bd on top
							case g is
								when 0 =>
									Bhat_addr <= conv_std_logic_vector(i, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_dto <= Bd(i); 	-- Bd(i)(j) if several inputs
									Bhat_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=1) then
								g:=0;
								j := j+1;
								if(j=n_u) then
									j:=0;
									i := i+1;
									if(i=n_x) then
										i:=0;
										init_nextstate <= gen_bhat_2;
										g := 0;h := 0;i := 0;j := 0;k := 1;
									end if;
								end if;
							end if;
							init_state <= delay;
						
						when gen_bhat_2 =>	-- Complete column 1
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector((k-1)*n_x+i, Ahat_r) & conv_std_logic_vector(h, Ahat_c);
									Ahat_we <= "0";
								when 1 => --We should multiply
									float_a <= Ahat_dfrom;
									float_b <= Bd(h);		--Bd(h)(j) for several inputs
								when 2 => --Get value from Bhat where multiplication is "added to"
									Bhat_addr <= conv_std_logic_vector(k*n_x+i, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_we <= "0";
								when 3 => --We should add
									float_a <= float_mul_res;
									float_b <= Bhat_dfrom;
									float_op <= "000000";
								when 4 => --Write new value
									Bhat_addr <= conv_std_logic_vector(k*n_x+i, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_dto <= float_add_res;
									Bhat_we <= "1";
								when others => g := 0;
							end case;
							-- This makes this state into 5 nested for loops
							g := g+1;
							if(g=5) then
								g:=0;
								h := h+1;
								if(h=n_x) then
									h:=0;
									j := j+1;
									if(j=n_u) then
										j:=0;
										i:=i+1;
										if(i=n_x) then
											i:=0;
											k:=k+1;
											if(k=n_pred) then
												k:=0;
												init_nextstate <= gen_bhat_3;
												g := 0;h := 0;i := n_x;j := n_u;k := 0;
											end if;
										end if;
									end if;
								end if;
							end if;
							init_state <= delay;
						
						when gen_bhat_3 => -- Copy column1 to the rest, shifted down one matrix							
							case g is
								when 0 => --Get value from Bhat
									Bhat_addr <= conv_std_logic_vector(i-n_x, Bhat_r) & conv_std_logic_vector(j-n_u, Bhat_c);
									Bhat_we <= "0";
								when 1 => --Output this to the correct position
									Bhat_addr <= conv_std_logic_vector(i, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_we <= "1";
									Bhat_dto <= Bhat_dfrom;
								when others => g := 0;
							end case;
							g := g+1;
							if(g=2) then
								g:=0;
								i := i+1;
								if(i=n_x*n_pred) then
									i:=n_x;
									j := j+1;
									if(j=n_u*n_pred) then
										init_nextstate <= idle;
										MPC_state <= idle;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							init_state <= delay;
							
						when idle => init_state <= idle;	
						when delay => init_state <= init_nextstate;
					end case;

				--		/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
				--		|						Update x0 values (run model of system) 				|
				--		\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
				when get_x0 =>
					case MODEL_state is	
						-- This adds A*x to the integrator matrix		
						when calc_states_1 =>
											float_a <= Ad(i)(j);
											float_b <= x0(j);
											
											MODEL_nextstate <= calc_states_2; 
											MODEL_state <= delay;
											
						when calc_states_2 =>
											float_a <= float_mul_res;	
											float_b <= x_int(i);
											if(j=0) then
												float_b <= "00000000000000000000000000000000";
											end if;
											float_op <= "000000";
											
											MODEL_nextstate <= calc_states_3; 
											MODEL_state <= delay;
									
						when calc_states_3 => 
											x_int(i) := float_add_res;
											if(j=n_x-1) then 
												j := 0;
												if(i=n_x-1) then
													i := 0;
													MODEL_state <= calc_states_4;
												else 
													i := i+1;
													MODEL_state <= calc_states_1;
												end if;
											else 
												j := j+1;
												MODEL_state <= calc_states_1;
											end if;
							
						-- This adds B*u to the state matrix		
						when calc_states_4 => 
											float_a <= Bd(i);
											float_b <= Out_Value_float;
											
											MODEL_nextstate <= calc_states_5; 
											MODEL_state <= delay;	

						when calc_states_5 =>
											float_a <= float_mul_res;
											float_b <= x_int(i);
											float_op <= "000000";
											
											MODEL_nextstate <= calc_states_6; 
											MODEL_state <= delay;

						when calc_states_6 => 
											x_int(i) := float_add_res;
											if(i=n_x-1) then
												i := 0;
												MODEL_state <= calc_states_7;
											else 
												i := i+1;
												MODEL_state <= calc_states_4;
											end if;	
						
						-- This adds L*(x_real-x_model) to the x_dot matrix
						when calc_states_7 =>
											std_to_float <= "0000" & P_Value;
											
											MODEL_nextstate <= calc_states_8; 
											MODEL_state <= delay;
						when calc_states_8 =>
											float_a <= float_gen;
											float_b <= "01000001100000000000000000000000"; --Divide by 16 to get "real" value
											
											MODEL_nextstate <= calc_states_9; 
											MODEL_state <= delay;
						when calc_states_9 =>
											float_a <= float_div_res;
											float_b <= x0(1);					--x0(1) is the only measured state
											float_op <= "000001";			--x_real-x_model
											
											MODEL_nextstate <= calc_states_10; 
											MODEL_state <= delay;
						when calc_states_10 => 
											float_a <= L_FB(i);
											float_b <= float_add_res;
											
											MODEL_nextstate <= calc_states_11; 
											MODEL_state <= delay;	

						when calc_states_11 =>
											float_a <= float_mul_res;
											float_b <= x_int(i);
											float_op <= "000000";
											
											MODEL_nextstate <= calc_states_12; 
											MODEL_state <= delay;

						when calc_states_12 => 
											x_int(i) := float_add_res;
											if(i=n_x-1) then
												i := 0;
												MODEL_state <= calc_states_13;
											else 
												i := i+1;
												MODEL_state <= calc_states_7;
											end if;	
																
						when calc_states_13 => 
											x0(i) := x_int(i);
											if(i=n_x-1) then	--We have gotten all the values
												g := 0;h := 0;i := 0;j := 0;k := 0;
												MODEL_state <= calc_states_1;
												MPC_state <= create_obj;
												obj_state <= gen_xdev_1;
											else 
												i := i+1;
												MODEL_state <= calc_states_13;
											end if;		

						when delay => 	MODEL_state <= MODEL_nextstate;
						when others => 	MODEL_state <= calc_states_1;
					end case;


				--		/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
				--		|						Generate the objective function						  |
				--		\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
				when create_obj =>
					counter := counter +1;
					case obj_state is
						when gen_xdev_1 =>	--Xdev=Ahat*x0;
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector(i, Ahat_r) & conv_std_logic_vector(j, Ahat_c);
									Ahat_we <= "0";
									Xdev_addr <= conv_std_logic_vector(i, Xdev_r);
									Xdev_we <= "0";
								when 1 => --We should multiply Ahat(i,j)*x0(j)
									float_a <= Ahat_dfrom;
									float_b <= x0(j);
								when 2 => --We should add
									float_a <= float_mul_res;
									float_b <= Xdev_dfrom;
									if(j=0) then
										float_b <= "00000000000000000000000000000000";
									end if;
									float_op <= "000000";
								when 3 => --Write new value
									Xdev_addr <= conv_std_logic_vector(i, Xdev_r);
									Xdev_dto <= float_add_res;
									Xdev_we <= "1";	
								when others => g := 0;
							end case;
						
							obj_nextstate <= gen_xdev_1;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+1;
									if(i=n_x*n_pred) then
										obj_nextstate <= gen_xdev_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
										Q_C_max := "00111111100000000000000000000000";
										J_G_max := "00111111100000000000000000000000";
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_xdev_2 =>	--Xdev=Ahat*x0-xref;
							case g is
								when 0 => --Get value from Xdev
									Xdev_addr <= conv_std_logic_vector(i+j, Xdev_r);
									Xdev_we <= "0";
								when 1 => --We should subtract
									float_a <= Xdev_dfrom;
									float_b <= Xref(j);
									float_op <= "000001";
								when 2 => --Write new value
									Xdev_addr <= conv_std_logic_vector(i+j, Xdev_r);
									Xdev_dto <= float_add_res;
									Xdev_we <= "1";
								when others => g := 0;
							end case;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+n_x;
									if(i=n_x*n_pred) then
										obj_nextstate <= gen_J1_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_J1_1 =>
							case g is
								when 0 => --Insert -1 diagonally
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(i, J1_c);
									J1_dto <= "10111111100000000000000000000000"; -- -1
									J1_we <= "1";
								when 1 => --Insert 1 diagonally
									J1_addr <= conv_std_logic_vector(i+n_pred*n_u, J1_r) & conv_std_logic_vector(i, J1_c);
									J1_dto <= "00111111100000000000000000000000"; -- 1
									J1_we <= "1";
								when others => g := 0;
							end case;
							g := g+1;
							if(g=2) then
								g:=0;
								i := i+1;
								if(i=n_pred*n_u) then
									obj_nextstate <= gen_J1_2;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_J1_2	=>				
							case g is
								when 0 => --Read from Bhat
									Bhat_addr <= conv_std_logic_vector(i, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_we <= "0";
								when 1 => --Insert into J1
									J1_addr <= conv_std_logic_vector(i+2*n_pred*n_u+n_pred*n_x, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_dto <= Bhat_dfrom;
									J1_we <= "1";
									float_a <= Bhat_dfrom;
									float_b <= "10111111100000000000000000000000";	-- Multiply with -1
								when 2 => --Insert this one into J1 as well
									J1_addr <= conv_std_logic_vector(i+2*n_pred*n_u, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_dto <= float_mul_res;
									J1_we <= "1";
									--Find largest value for making range -1 to 1
									float_a <= "0" & float_mul_res(30 downto 0);
									float_b <= J_G_max;
									temp_max := float_mul_res;
								when 3 =>
									--Additional delay for comparator
								when 4 =>
									if(float_comp_res = "1") then
										J_G_max := "0" & temp_max(30 downto 0);
									end if;								
								when others => g := 0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=n_pred*n_u) then
									j:=0;
									i := i+1;
									if(i=n_pred*n_x) then
										obj_nextstate <= gen_G1_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_G1_1 =>
							case g is
								when 0 => --Get -umin
									float_a <= umin(j);
									float_b <= "10111111100000000000000000000000";	-- Multiply with -1
								when 1 => --Write to G1
									G1_addr <= conv_std_logic_vector(i+j, G1_r);
									G1_dto <= float_mul_res;
									G1_we <= "1";
								when 2 => --Insert into G1
									G1_addr <= conv_std_logic_vector(n_pred*n_u+i+j, G1_r);
									G1_dto <= umax(j);
									G1_we <= "1";
								--Find largest value for making range -1 to 1
									float_a <= "0" & umax(j)(30 downto 0);
									float_b <= J_G_max;
									temp_max := umax(j);
								when 3 =>
									--Additional delay for comparator
								when 4 =>
									if(float_comp_res = "1") then
										J_G_max := "0" & temp_max(30 downto 0);
									end if;	
								when others => g:=0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=n_u) then
									j:=0;
									i := i+n_u;
									if(i=n_u*n_pred) then
										obj_nextstate <= gen_G1_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_G1_2 =>
							case g is
								when 0 => --Get Xdev
									Xdev_addr <= conv_std_logic_vector(i+j, Xdev_r);
									Xdev_we <= "0";
									float_a <= Xref(j);
									float_b <= xmin(j);
									float_op <= "000001";   -- Subtract
								when 1 => 
									float_a <= float_add_res;
									float_b <= Xdev_dfrom;
									float_op <= "000000";   -- Add
								when 2 => --Insert into G1
									G1_addr <= conv_std_logic_vector(i+j+2*N_pred*n_u, G1_r);
									G1_dto <= float_add_res;
									G1_we <= "1";
									--Find largest value for making range -1 to 1
									float_a <= "0" & float_add_res(30 downto 0);
									float_b <= J_G_max;
									temp_max := float_add_res;
								when 3 =>
									--Additional delay for comparator
								when 4 =>
									if(float_comp_res = "1") then
										J_G_max := "0" & temp_max(30 downto 0);
									end if;	
								when 5 => 
									Xdev_addr <= conv_std_logic_vector(i+j, Xdev_r);
									Xdev_we <= "0";
									float_a <= xmax(j);
									float_b <= Xref(j);
									float_op <= "000001";   -- Subtract
								when 6 => 
									float_a <= float_add_res;
									float_b <= Xdev_dfrom;
									float_op <= "000001";   -- Subtract again
								when 7 => --Insert into G1
									G1_addr <= conv_std_logic_vector(i+j+2*N_pred*n_u+N_pred*n_x, G1_r);
									G1_dto <= float_add_res;
									G1_we <= "1";
									--Find largest value for making range -1 to 1
									float_a <= "0" & float_add_res(30 downto 0);
									float_b <= J_G_max;
									temp_max := float_add_res;
								when 8 =>
									--Additional delay for comparator
								when 9 =>
									if(float_comp_res = "1") then
										J_G_max := "0" & temp_max(30 downto 0);
									end if;	
								when others => g:=0;
							end case;
							obj_nextstate <= gen_G1_2;
							g := g+1;
							if(g=10) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+n_x;
									if(i=N_pred*n_x) then
										obj_nextstate <= gen_Q_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;

						when gen_Q_1 => --Temp1=Qhat*Bhat;
							case g is
								when 0 => --Get Bhat
									Bhat_addr <= conv_std_logic_vector(i+k, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_we <= "0";
								when 1 => 
									float_a <= Bhat_dfrom;
									float_b <= Qx(k);
								when 2 =>
									Temp1_addr <= conv_std_logic_vector(i+k, Temp1_r) & conv_std_logic_vector(j, Temp1_c);
									Temp1_dto <= float_mul_res;
									Temp1_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								g:=0;
								k := k+1;
								if(k=n_x) then
									k:=0;
									j := j+1;
									if(j=N_pred*n_u) then
										j:=0;
										i:=i+n_x;
										if(i=N_pred*n_x) then
											obj_nextstate <= gen_Q_2;
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_Q_2 => --Q=Bhat'*Temp1;
							case g is
								when 0 => --Get Values
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_we <= "0";
									Bhat_addr <= conv_std_logic_vector(k, Bhat_r) & conv_std_logic_vector(j, Bhat_c);
									Bhat_we <= "0";
									Temp1_addr <= conv_std_logic_vector(k, Temp1_r) & conv_std_logic_vector(i, Temp1_c);
									Temp1_we <= "0";
								when 1 => 
									float_a <= Bhat_dfrom;
									float_b <= Temp1_dfrom;
								when 2 =>
									float_a <= float_mul_res;
									float_b <= Q_dfrom;
									if(k=0) then
										float_b <= "00000000000000000000000000000000";
									end if;
									float_op <= "000000";
								when 3 =>
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_dto <= float_add_res;
									Q_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								k := k+1;
								if(k=n_x*n_pred) then
									k:=0;
									j := j+1;
									if(j=N_pred*n_u) then
										j:=0;
										i:=i+1;
										if(i=N_pred*n_u) then
											obj_nextstate <= gen_Q_3;
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_Q_3 =>  --Q(i+j,i+j)=Q(i+j,i+j)+Qu(j);
							case g is
								when 0 => --Get Values
									Q_addr <= conv_std_logic_vector(i+j, Q_r) & conv_std_logic_vector(i+j, Q_c);
									Q_we <= "0";
								when 1 => 
									float_a <= Q_dfrom;
									float_b <= Qu(j);
									float_op <= "000000";
								when 2 =>
									Q_addr <= conv_std_logic_vector(i+j, Q_r) & conv_std_logic_vector(i+j, Q_c);
									Q_dto <= float_add_res;
									Q_we <= "1";
									float_a <= "0" & float_add_res(30 downto 0);
									float_b <= Q_C_max;
									temp_max := float_add_res;
								when 3 =>
									--Additional delay for comparator
								when 4 =>
									if(float_comp_res = "1") then
										Q_C_max := "0" & temp_max(30 downto 0);
									end if;
								when others => g:=0;
							end case;
							obj_state <= delay;
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=n_u) then
									j:=0;
									i := i+n_u;
									if(i=N_pred*n_u) then
										obj_nextstate <= gen_C_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
						
						when gen_C_1 =>	--c(i)=c(i)+Xdev(j)*temp1(j,i);
							case g is
								when 0 => --Get Values									
									C_addr <= conv_std_logic_vector(i, C_r);
									C_we <= "0";
									Xdev_addr <= conv_std_logic_vector(j, Xdev_r);
									Xdev_we <= "0";
									Temp1_addr <= conv_std_logic_vector(j, Temp1_r) & conv_std_logic_vector(i, Temp1_c);
									Temp1_we <= "0";
								when 1 => 
									float_a <= Xdev_dfrom;
									float_b <= Temp1_dfrom;
								when 2 => 
									float_a <= float_mul_res;
									float_b <= C_dfrom;
									if(j=0) then
										float_b <= "00000000000000000000000000000000";
									end if;
									float_op <= "000000";
								when 3 =>
									C_addr <= conv_std_logic_vector(i, C_r);
									C_dto <= float_add_res;
									C_we <= "1";
									
									float_a <= "0" & float_add_res(30 downto 0);
									float_b <= Q_C_max;
									temp_max := float_add_res;
								when 4 =>
									--Additional delay for comparator
								when 5 =>
									if(j=(n_x*n_pred)-1) then
										if(float_comp_res="1") then
											Q_C_max := "0" & temp_max(30 downto 0);
										end if;
									end if;
								when others => g:=0;
							end case;
							g := g+1;
							if(g=6) then
								g:=0;
								j := j+1;
								if(j=n_x * n_pred) then
									j:=0;
									i := i+1;
									if(i=N_pred*n_u) then
										obj_nextstate <= range_Q_C;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when range_Q_C =>
							case g is
								when 0 => --Get Values									
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_we <= "0";
									C_addr <= conv_std_logic_vector(i, C_r);
									C_we <= "0";
								when 1 =>									
									float_a <= Q_dfrom;
									float_b <= Q_C_max;
								when 2 =>
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_dto <= float_div_res;
									Q_we <= "1";
									
									float_a <= C_dfrom;
									float_b <= Q_C_max;
								when 3 =>
									if(j=N_pred*n_u-1) then
										C_addr <= conv_std_logic_vector(i, C_r);
										C_dto <= float_div_res;
										C_we <= "1";
									end if;
								when others => g:=0;
							end case;
							obj_state <= delay;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j:=0;
									i := i+1;
									if(i=N_pred*n_u) then
										obj_nextstate <= range_J_G;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							
						when range_J_G =>
							case g is
								when 0 => --Get Values									
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_we <= "0";
									G1_addr <= conv_std_logic_vector(i, G1_r);
									G1_we <= "0";
								when 1 =>									
									float_a <= J1_dfrom;
									float_b <= J_G_max;
								when 2 =>
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_dto <= float_div_res;
									J1_we <= "1";
									float_a <= G1_dfrom;
									float_b <= J_G_max;
								when 3 =>
									if(j=N_pred*n_u-1) then
										G1_addr <= conv_std_logic_vector(i, G1_r);
										G1_dto <= float_div_res;
										G1_we <= "1";
									end if;
								when others => g:=0;
							end case;
							obj_state <= delay;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j:=0;
									i := i+n_u;
									if(i=N_pred*n_c) then
										obj_nextstate <= gen_ZLT;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							
						when gen_ZLT =>
							case g is
								when 0 => --Get Values									
									Z_addr <= conv_std_logic_vector(i, Z_r);
									Z_dto <= "00000000000000000000000000000000";	-- Insert zeros
									Z_we <= "1";
									L_addr <= conv_std_logic_vector(j, L_r);
									L_dto <= "00111111100000000000000000000000";	-- Insert ones
									L_we <= "1";
									T_addr <= conv_std_logic_vector(j, T_r);
									T_dto <= "00111111100000000000000000000000";	-- Insert ones
									T_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=1) then
								g:=0;
								i := i+1;
								if(i=n_u * n_pred) then
									i:=0;
								end if;
								j := j+1;
								if(j=n_c * n_pred) then	-- This should be completed last
									obj_nextstate <= gen_xdev_1;
									MPC_state <= calc_min;
									calc_min_state <= gen_G2_1;
									g := 0;h := 0;i := 0;j := 0;k := 0;
									my := "00111111100000000000000000000000";
								end if;
							end if;
							obj_state <= delay;
						
						when delay => obj_state <= obj_nextstate;
					end case; -- Finished generating object function

				--		/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
				--		|						Find the minimizing vector									|
				--		\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
				when calc_min =>
					counter := counter +1;
					case calc_min_state is
						when gen_G2_1 =>	--G2(i)=-t(i)/l(i);
							case g is
								when 0 =>
									T_addr <= conv_std_logic_vector(i, T_r);
									T_we <= "0";
									L_addr <= conv_std_logic_vector(i, L_r);
									L_we <= "0";
								when 1 =>	--Divide these
									float_a <= T_dfrom;
									float_b <= L_dfrom;
								when 2 =>	--Insert -(result)
									G2_addr <= conv_std_logic_vector(i, G2_r);
									G2_dto(30 downto 0) <= float_div_res(30 downto 0);
									G2_dto(31) <= not(float_div_res(31));
									G2_we <= "1";									
								when others => g:=0;
							end case;
							calc_min_nextstate <= gen_G2_1;
							g := g+1;
							if(g=3) then
								g:=0;
								i := i+1;
								if(i=n_c * n_pred) then
									calc_min_nextstate <= gen_R1_1;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_R1_1 => --r1(i)=-c(i)-Q(i,j)*z(j);
							case g is
								when 0 =>							
									C_addr <= conv_std_logic_vector(i, C_r);
									C_we <= "0";
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_we <= "0";
									Z_addr <= conv_std_logic_vector(j, Z_r);
									Z_we <= "0";
								when 1 =>
									if(j=0) then
										R1_addr <= conv_std_logic_vector(i, R1_r);
										R1_dto(30 downto 0) <= C_dfrom(30 downto 0);
										R1_dto(31) <= not(C_dfrom(31));
										R1_we <= "1";
									end if;
								when 2 =>
									R1_addr <= conv_std_logic_vector(i, R1_r);
									R1_we <= "0";
									float_a <= Q_dfrom;
									float_b <= Z_dfrom;
								when 3 =>
									float_a <= R1_dfrom;
									float_b <= float_mul_res;
									float_op <= "000001";
								when 4 =>								
									R1_addr <= conv_std_logic_vector(i, R1_r);
									R1_dto <= float_add_res;
									R1_we <= "1";
								when others => g:=0;
							end case;	
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=n_pred) then
									j:=0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= gen_R1_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_R1_2 => --r1(i)=r1(i)-J(j,i)*l(j);
							case g is
								when 0 =>
									R1_addr <= conv_std_logic_vector(i, R1_r);
									R1_we <= "0";
									J1_addr <= conv_std_logic_vector(j, J1_r) & conv_std_logic_vector(i, J1_c);
									J1_we <= "0";
									L_addr <= conv_std_logic_vector(j, L_r);
									L_we <= "0";
								when 1 =>
									float_a <= J1_dfrom;
									float_b <= L_dfrom;
								when 2 =>
									float_a <= R1_dfrom;
									float_b <= float_mul_res;
									float_op <= "000001";
								when 3 =>							
									R1_addr <= conv_std_logic_vector(i, R1_r);
									R1_dto <= float_add_res;
									R1_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=n_pred*n_c) then
									j:=0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= gen_R2_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
						
						when gen_R2_1 =>  --r2(i)=g1(i)-rho*my/l(i);
							case g is
								when 0 =>
									G1_addr <= conv_std_logic_vector(i, G1_r);
									G1_we <= "0";
									L_addr <= conv_std_logic_vector(i, L_r);
									L_we <= "0";
								when 1 =>
									float_a <= rho;
									float_b <= my;
								when 2 =>
									float_a <= float_mul_res;
									float_b <= L_dfrom;
								when 3 =>
									float_a <= G1_dfrom;
									float_b <= float_div_res;
									float_op <= "000001";
								when 4 =>
									R2_addr <= conv_std_logic_vector(i, R2_r);
									R2_dto <= float_add_res;
									R2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_c) then
									calc_min_nextstate <= gen_R2_2;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_R2_2 =>  --r2(i)=r2(i)-J(i,j)*z(j);
							case g is
								when 0 =>									
									R2_addr <= conv_std_logic_vector(i, R2_r);
									R2_we <= "0";
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_we <= "0";
									Z_addr <= conv_std_logic_vector(j, Z_r);
									Z_we <= "0";
								when 1 =>
									float_a <= J1_dfrom;
									float_b <= Z_dfrom;
								when 2 =>
									float_a <= R2_dfrom;
									float_b <= float_mul_res;
									float_op <= "000001";
								when 3 =>
									R2_addr <= conv_std_logic_vector(i, R2_r);
									R2_dto <= float_add_res;
									R2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=N_pred) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_c) then
										calc_min_nextstate <= gen_D_Z_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_D_Z_1 =>  --temp2(i,j)=J(j,i)/(G2(j));
							case g is
								when 0 =>									
									J1_addr <= conv_std_logic_vector(j, J1_r) & conv_std_logic_vector(i, J1_c);
									J1_we <= "0";
									G2_addr <= conv_std_logic_vector(j, G2_r);
									G2_we <= "0";
								when 1 =>
									float_a <= J1_dfrom;
									float_b <= G2_dfrom;
								when 2 =>
									Temp2_addr <= conv_std_logic_vector(i, Temp2_r) & conv_std_logic_vector(j, Temp2_c);
									Temp2_dto <= float_div_res;
									Temp2_we <= "1";
									
									--Let's clear temp3 as well
									Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_dto <= "00000000000000000000000000000000";
									Temp3_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_c*N_pred) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= gen_D_Z_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
						
						when gen_D_Z_2 => --temp3(i,j)=temp3(i,j)+temp2(i,k)*J(k,j);
							case g is
								when 0 =>	
									Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_we <= "0";
									Temp2_addr <= conv_std_logic_vector(i, Temp2_r) & conv_std_logic_vector(k, Temp2_c);
									Temp2_we <= "0";
									J1_addr <= conv_std_logic_vector(k, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_we <= "0";						
								when 1 =>
									float_a <= Temp2_dfrom;
									float_b <= J1_dfrom;
								when 2 =>
									float_a <= float_mul_res;
									float_b <= Temp3_dfrom;
									float_op <= "000000";
								when 3 =>
									Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_dto <= float_add_res;
									Temp3_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								k := k+1;
								if(k=n_c*n_pred) then
									k := 0;
									j := j+1;
									if(j=n_u*N_pred) then
										j := 0;
										i := i+1;
										if(i=N_pred*n_u) then
											calc_min_nextstate <= gen_D_Z_3;
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_D_Z_3 => --temp3(i,j)=Q(i,j)-temp3(i,j);
							case g is
								when 0 =>	
									Q_addr <= conv_std_logic_vector(i, Q_r) & conv_std_logic_vector(j, Q_c);
									Q_we <= "0";
									Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_we <= "0";	
									 -- Also create Temp4=eye(np*nu) and clear 
									Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(j, Temp4_c);
									if(i=j) then
										Temp4_dto <= "00111111100000000000000000000000";
									else
										Temp4_dto <= "00000000000000000000000000000000";
									end if;
									Temp4_we <= "1";
									R_addr <= conv_std_logic_vector(i, R_r);
									R_dto <= "00000000000000000000000000000000";
									R_we <= "1";
								when 1 =>
									float_a <= Q_dfrom;
									float_b <= Temp3_dfrom;
									float_op <= "000001";
								when 2 =>
									Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_dto <= float_add_res;
									Temp3_we <= "1";
									
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= gen_INV_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_INV_1 =>
							case loopnum is
								when 0 =>
									case g is
										when 0 =>
											Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
											Temp3_we <= "0";
										when 1 =>
											n := temp3_dfrom;
										when others => g := 0;
									end case;
								when 1 =>
									case g is
										when 0 =>	
											Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
											Temp3_we <= "0";
											Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(j, Temp4_c);
											Temp4_we <= "0";
										when 1 =>
											float_a <= Temp3_dfrom;
											float_b <= n;
										when 2 =>
											Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
											Temp3_dto <= float_div_res;
											Temp3_we <= "1";
											float_a <= Temp4_dfrom;
											float_b <= n;
										when 3 =>
											Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(j, Temp4_c);
											Temp4_dto <= float_div_res;
											Temp4_we <= "1";
										when others => g:=0;
									end case;
								when 2 =>
									case g is
										when 0 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
											Temp3_we <= "0";
										when 1 =>
											R_addr <= conv_std_logic_vector(j, R_r);
											R_dto(30 downto 0) <= Temp3_dfrom(30 downto 0);
											R_dto(31) <= not(Temp3_dfrom(31));
											R_we <= "1";
										when others => g:=0;
									end case;
								when 3 =>
									R_addr <= conv_std_logic_vector(i, R_r);
									R_dto <= "00000000000000000000000000000000";
									R_we <= "1";
								when 4 =>
									case g is
										when 0 =>	
											Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_we <= "0";
											Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_we <= "0";
											R_addr <= conv_std_logic_vector(j, R_r);
											R_we <= "0";
										when 1 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_we <= "0";
											float_a <= Temp3_dfrom;
											float_b <= R_dfrom;
										when 2 =>
											float_a <= Temp3_dfrom;
											float_b <= float_mul_res;
											float_op <= "000000";
										when 3 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_dto <= float_add_res;
											Temp3_we <= "1";
											Temp4_addr <= conv_std_logic_vector(j, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_we <= "0";
											float_a <= Temp4_dfrom;
											float_b <= R_dfrom;
										when 4 =>
											float_a <= Temp4_dfrom;
											float_b <= float_mul_res;
											float_op <= "000000";
										when 5 =>
											Temp4_addr <= conv_std_logic_vector(j, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_dto <= float_add_res;
											Temp4_we <= "1";
										when others => g:=0;
									end case;
								when others => loopnum := 0;
							end case;
							case loopnum is
								when 0 =>
									g := g+1;
									if(g=2) then
										g := 0;
										loopnum := loopnum+1;
									end if;
								when 1 =>
									g := g+1;
									if(g=4) then
										g := 0;
										j := j+1;
										if(j=N_pred*n_u) then
											j := 0;
											loopnum := loopnum+1;
										end if;
									end if;
								when 2 =>
									g := g+1;
									if(g=2) then
										g := 0;
										j := j+1;
										if(j=N_pred*n_u) then
											j := 0;
											loopnum := loopnum+1;
										end if;
									end if;
								when 3 =>
									loopnum := loopnum+1;
									j := i+1;
									if(j=N_pred*n_u) then
										calc_min_nextstate <= gen_INV_2;
										g := 0;h := 0;i := n_pred*n_u;j := 0;k := 0;loopnum:=0;
									end if;
								when 4 =>
									g := g+1;
									if(g=6) then
										g := 0;
										k := k+1;
										if(k=N_pred*n_u) then
											k := 0;
											j := j+1;
											if(j=N_pred*n_u) then
												j := 0;
												loopnum := 0;
												i := i+1;
											end if;
										end if;
									end if;	
								when others => loopnum := 0;
							end case;
							calc_min_state <= delay;
						
						when gen_INV_2 =>
							case loopnum is							
								when 0 =>
									case g is
										when 0 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
											Temp3_we <= "0";
										when 1 =>
											R_addr <= conv_std_logic_vector(j, R_r);
											R_dto(30 downto 0) <= Temp3_dfrom(30 downto 0);
											R_dto(31) <= not(Temp3_dfrom(31));
											R_we <= "1";
										when others => g:=0;
									end case;
								when 1 =>
									R_addr <= conv_std_logic_vector(i, R_r);
									R_dto <= "00000000000000000000000000000000";
									R_we <= "1";
								when 2 =>
									case g is
										when 0 =>	
											Temp3_addr <= conv_std_logic_vector(i, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_we <= "0";
											Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_we <= "0";
											R_addr <= conv_std_logic_vector(j, R_r);
											R_we <= "0";
										when 1 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_we <= "0";
											float_a <= Temp3_dfrom;
											float_b <= R_dfrom;
										when 2 =>
											float_a <= Temp3_dfrom;
											float_b <= float_mul_res;
											float_op <= "000000";
										when 3 =>
											Temp3_addr <= conv_std_logic_vector(j, Temp3_r) & conv_std_logic_vector(k, Temp3_c);
											Temp3_dto <= float_add_res;
											Temp3_we <= "1";
											Temp4_addr <= conv_std_logic_vector(j, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_we <= "0";
											float_a <= Temp4_dfrom;
											float_b <= R_dfrom;
										when 4 =>
											float_a <= Temp4_dfrom;
											float_b <= float_mul_res;
											float_op <= "000000";
										when 5 =>
											Temp4_addr <= conv_std_logic_vector(j, Temp4_r) & conv_std_logic_vector(k, Temp4_c);
											Temp4_dto <= float_add_res;
											Temp4_we <= "1";
										when others => g:=0;
									end case;
								when others => loopnum := 0;
							end case;
							case loopnum is
								when 0 =>
									g := g+1;
									if(g=2) then
										g := 0;
										j := j+1;
										if(j=N_pred*n_u) then
											j := 0;
											loopnum := loopnum+1;
										end if;
									end if;
								when 1 =>
									loopnum := loopnum+1;
									j := i-1;
								when 2 =>
									g := g+1;
									if(g=6) then
										g := 0;
										k := k+1;
										if(k=N_pred*n_u) then
											k := 0;
											j := j-1;
											if(j=-1) then
												loopnum := 0;
												i := i-1;
												if(i=0) then
													calc_min_nextstate <= gen_D_Z_4;
													g := 0;h := 0;i := 0;j := 0;k := 0;loopnum:=0;
												end if;
											end if;
										end if;
									end if;	
								when others => loopnum := 0;
							end case;
							calc_min_state <= delay;
							
						when gen_D_Z_4 => --temp3(0,i)=r1(i);
							case g is
								when 0 =>								
									R1_addr <= conv_std_logic_vector(i, 4);
									R1_we <= "0";
								when 1 =>
									Temp3_addr <= conv_std_logic_vector(0, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
									Temp3_dto <= R1_dfrom;
									Temp3_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=2) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_u) then
									calc_min_nextstate <= gen_D_Z_5;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
						
						when gen_D_Z_5 => --temp3(0,i)=temp3(0,i)-temp2(i,j)*r2(j);
							case g is
								when 0 =>
									Temp3_addr <= conv_std_logic_vector(0, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
									Temp3_we <= "0";
									Temp2_addr <= conv_std_logic_vector(i, Temp2_r) & conv_std_logic_vector(j, Temp2_c);
									Temp2_we <= "0";
									R2_addr <= conv_std_logic_vector(j, R2_r);
									R2_we <= "0";
								when 1 =>
									float_a <= Temp2_dfrom;
									float_b <= R2_dfrom;
								when 2 =>
									float_a <= Temp3_dfrom;
									float_b <= float_mul_res;
									float_op <= "000001";
								when 3 =>
									Temp3_addr <= conv_std_logic_vector(0, Temp3_r) & conv_std_logic_vector(i, Temp3_c);
									Temp3_dto <= float_add_res;
									Temp3_we <= "1";
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= "00000000000000000000000000000000";
									Temp2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_c) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= clr_Temp2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when clr_Temp2 => --Clear variables only
							case g is
								when 0 =>
									Temp2_addr <= conv_std_logic_vector(0, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= "00000000000000000000000000000000";
									Temp2_we <= "1";
								when 1 =>
									Temp2_addr <= conv_std_logic_vector(1, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= "00000000000000000000000000000000";
									Temp2_we <= "1";
								when 2 =>
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= "00000000000000000000000000000000";
									Temp2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_c) then
									calc_min_nextstate <= gen_D_Z_6;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_D_Z_6 => --d_z(i)=d_z(i)+temp4(i,j)*temp3(0,j);
							case g is
								when 0 =>
									Temp4_addr <= conv_std_logic_vector(i, Temp4_r) & conv_std_logic_vector(j, Temp4_c);
									Temp4_we <= "0";
									Temp3_addr <= conv_std_logic_vector(0, Temp3_r) & conv_std_logic_vector(j, Temp3_c);
									Temp3_we <= "0";
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
								when 1 =>
									float_a <= Temp4_dfrom;
									float_b <= Temp3_dfrom;
								when 2 =>
									float_a <= Temp2_dfrom;
									float_b <= float_mul_res;
									float_op <= "000000";
								when 3 =>
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= float_add_res;
									Temp2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_u) then
										calc_min_nextstate <= gen_Z_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_Z_1 => --z(i)=z(i)+alpha*d_z(i);
							case g is
								when 0 =>
									Z_addr <= conv_std_logic_vector(i, Z_r);
									Z_we <= "0";
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
								when 1 =>
									float_a <= Temp2_dfrom;
									float_b <= alpha;
								when 2 =>
									float_a <= Z_dfrom;
									float_b <= float_mul_res;
									float_op <= "000000";
								when 3 =>
									Z_addr <= conv_std_logic_vector(i, Z_r);
									Z_dto <= float_add_res;
									Z_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_u) then
									calc_min_nextstate <= gen_L_1;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_L_1 => --temp2(0,i)=temp2(0,i)+J(i,j)*d_z(j)/G2(i);
							case g is
								when 0 =>
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(j, Temp2_c);
									Temp2_we <= "0";
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_we <= "0";
									G2_addr <= conv_std_logic_vector(i, G2_r);
									G2_we <= "0";
								when 1 =>
									float_a <= J1_dfrom;
									float_b <= Temp2_dfrom;
								when 2 =>
									float_a <= float_mul_res;
									float_b <= G2_dfrom;
									Temp2_addr <= conv_std_logic_vector(0, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
								when 3 =>
									float_a <= Temp2_dfrom;
									float_b <= float_div_res;
									float_op <= "000000";
								when 4 =>
									Temp2_addr <= conv_std_logic_vector(0, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= float_add_res;
									Temp2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_c) then
										calc_min_nextstate <= gen_L_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_L_2 => --l(i)=l(i) + alpha* ( r2(i)/G2(i)-temp2(0,i) );
							case g is
								when 0 =>
									R2_addr <= conv_std_logic_vector(i, R2_r);
									R2_we <= "0";
									G2_addr <= conv_std_logic_vector(i, G2_r);
									G2_we <= "0";
									Temp2_addr <= conv_std_logic_vector(0, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
									L_addr <= conv_std_logic_vector(i, L_r);
									L_we <= "0";
								when 1 =>
									float_a <= R2_dfrom;
									float_b <= G2_dfrom;
								when 2 =>
									float_a <= float_div_res;
									float_b <= Temp2_dfrom;
									float_op <= "000001";
								when 3 =>
									float_a <= float_add_res;
									float_b <= usedalpha;
								when 4 =>
									float_a <= float_mul_res;
									float_b <= L_dfrom;
									float_op <= "000000";
								when 5 =>						--Check if this is below zero
									tempvar := float_add_res;
									float_a <= "00000000000000000000000000000000";
									float_b <= float_add_res;
								when 6 =>
									--Delay for comparator
								when 7 =>
									if(float_comp_res = "1") then		--It was below zero, reduce alpha
										float_a <= usedalpha;
										float_b <= alpha; --usedalpha=usedalpha*alpha
									else
										usedalpha := alpha;
										g := 8; --Will be increased to 9
									end if;	
								when 8 =>
									usedalpha := float_mul_res;
									g := 0; --Will be increased to 1.
								when 9 =>
									L_addr <= conv_std_logic_vector(i, L_r);
									L_dto <= tempvar;
									L_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=10) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_c) then
									calc_min_nextstate <= gen_T_1;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
										
						when gen_T_1 => --temp2(1,i)=temp2(1,i)+J(i,j)*(z(j)+temp2(2,j));
							case g is
								when 0 =>
									Temp2_addr <= conv_std_logic_vector(2, Temp2_r) & conv_std_logic_vector(j, Temp2_c);
									Temp2_we <= "0";
									J1_addr <= conv_std_logic_vector(i, J1_r) & conv_std_logic_vector(j, J1_c);
									J1_we <= "0";
									Z_addr <= conv_std_logic_vector(j, Z_r);
									Z_we <= "0";
								when 1 =>
									float_a <= Z_dfrom;
									float_b <= Temp2_dfrom;
									float_op <= "000000";
								when 2 =>
									float_a <= float_add_res;
									float_b <= J1_dfrom;
									Temp2_addr <= conv_std_logic_vector(1, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
								when 3 =>
									float_a <= float_mul_res;
									float_b <= Temp2_dfrom;
									float_op <= "000000";
								when 4 =>
									Temp2_addr <= conv_std_logic_vector(1, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_dto <= float_add_res;
									Temp2_we <= "1";
								when others => g:=0;
							end case;
							g := g+1;
							if(g=5) then
								g:=0;
								j := j+1;
								if(j=N_pred*n_u) then
									j := 0;
									i := i+1;
									if(i=N_pred*n_c) then
										calc_min_nextstate <= gen_T_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_T_2 => --t(i)=t(i) + alpha*( g1(i)-t(i)-temp2(1,i) );
							case g is
								when 0 =>
									T_addr <= conv_std_logic_vector(i, T_r);
									T_we <= "0";
									G1_addr <= conv_std_logic_vector(i, G1_r);
									G1_we <= "0";
									Temp2_addr <= conv_std_logic_vector(1, Temp2_r) & conv_std_logic_vector(i, Temp2_c);
									Temp2_we <= "0";
								when 1 =>
									float_a <= G1_dfrom;
									float_b <= T_dfrom;
									float_op <= "000001";
								when 2 =>
									float_a <= float_add_res;
									float_b <= Temp2_dfrom;
									float_op <= "000001";
								when 3 =>
									float_a <= float_add_res;
									float_b <= usedalpha;
								when 4 =>
									float_a <= float_mul_res;
									float_b <= T_dfrom;
									float_op <= "000000";
								when 5 =>						--Check if this is below zero
									tempvar := float_add_res;
									float_a <= "00000000000000000000000000000000";
									float_b <= float_add_res;
								when 6 =>
									--Delay for comparator
								when 7 =>
									if(float_comp_res = "1") then		--It was below zero, reduce alpha
										float_a <= usedalpha;
										float_b <= alpha; --usedalpha=usedalpha*alpha
									else
										usedalpha := alpha;
										g := 8; --Will be increased to 9
									end if;
								when 8 =>
									usedalpha := float_mul_res;
									g := 0; --Will be increased to 1.
								when 9 =>
									T_addr <= conv_std_logic_vector(i, T_r);
									T_dto <= tempvar;
									T_we <= "1";
								when others => g:=0;	
							end case;
							g := g+1;
							if(g=10) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_c) then
									calc_min_nextstate <= gen_my_1;
									g := 0;h := 0;i := 0;j := 0;k := 0;
									my := "00000000000000000000000000000000";
								end if;
							end if;
							calc_min_state <= delay;
					
						when gen_my_1 => --my=my+t(i)*l(i);
							case g is
								when 0 =>
									T_addr <= conv_std_logic_vector(i, T_r);
									T_we <= "0";
									L_addr <= conv_std_logic_vector(i, L_r);
									L_we <= "0";
								when 1 =>
									float_a <= T_dfrom;
									float_b <= L_dfrom;
								when 2 =>
									float_a <= my;
									float_b <= float_mul_res;
									float_op <= "000000";
								when 3 =>
									my := float_add_res;
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								g:=0;
								i := i+1;
								if(i=N_pred*n_c) then
									calc_min_nextstate <= gen_my_2;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							calc_min_state <= delay;
							
						when gen_my_2 => --my=my/N_pred*n_c;
							case g is
								when 0 =>
									std_to_float <= conv_std_logic_vector(N_pred*n_c, 16);
								when 1 =>
									float_a <= my;
									float_b <= float_gen;
								when 2 =>
									my := float_div_res;
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								calc_min_nextstate <= gen_my_3;
								g := 0;h := 0;i := 0;j := 0;k := 0;
							end if;
							calc_min_state <= delay;
							
						when gen_my_3 => --Check if my is below limit and finish, or iterate more
							case g is
								when 0 =>
									float_a <= my;
									float_b <= mymax;
								when 1 =>
									--more delay
								when 2 =>
									if(float_comp_res = "1") then --my still too large
										calc_min_nextstate <= gen_G2_1;
										g := -1;h := 0;i := 0;j := 0;k := 0;
									end if;
								when others => g:=0;
							end case;
							g := g+1;
							if(g=3) then
								calc_min_nextstate <= gen_output;
								g := 0;h := 0;i := 0;j := 0;k := 0;
							end if;
							calc_min_state <= delay;
						
						when gen_output => --calculate output for the next ten samples
							--We have a 12 bit output vector, representing 0-3.3V. We say that motor voltage range is 0 to 255V,
							--giving us that the output value 4096 = 256V.
							--Thus, output value = 16 * motorvoltage
							case g is
								when 0 =>
									Z_addr <= conv_std_logic_vector(0, Z_r);
									Z_we <= "0";
									debug <= counter;
								when 1 =>
									counter := "00000000000000000000000000000000";
									Out_value_float := Z_dfrom;
									--Sometimes, when changing reference, z_dfrom get's messed up. This "protects" it.
									--if(Out_array(Outvar) = "01111111110000000000000000000000") then
									if(Out_value_float = "01111111110000000000000000000000") then
										Out_value_float := prev_Z_dfrom;
									end if;
									
									float_a <= Out_value_float;
									float_b <= "01000001100000000000000000000000"; --Multiply with 16 to get DAC value
								when 2 =>
									float_to_std <= float_mul_res;
								when 3 =>
									Out_value <= std_gen(11 downto 0);
									prev_Z_dfrom := Out_value_float;
									MPC_done <= '1';
								when others => g:=0;
							end case;
							g := g+1;
							if(g=4) then
								calc_min_nextstate <= gen_G2_1;
								MPC_state <= idle;
								g := 0;h := 0;i := 0;j := 0;k := 0;
							end if;
							calc_min_state <= delay;
						
						when delay => calc_min_state <= calc_min_nextstate;
					end case; -- Finished calculating the minimizer
				
				when idle => -- Wait here for new sample
					if(SYS_state = calc_MPC) then	
						g := 0;h := 0;i := 0;j := 0;k := 0;
						MPC_state <= get_x0;
						MODEL_state <= calc_states_1;
						MPC_done <= '0';
					end if;

				when others => MPC_state <= init;
			end case;
		end if;
	end process MPC_proc;
end architecture basic;