--MPC block-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;
use work.mpc_package.all;

entity MPC is
   Port ( clk 			 : in std_logic;
			 Ref 			 : in std_logic_vector(7 downto 0);
			 x0 			 : in array1;
			 MPC_u1		 : out std_logic_vector(11 downto 0);
			 debug		 : out std_logic_vector(31 downto 0);
			 float_a 		: out std_logic_vector(31 downto 0);
			 float_b 		: out std_logic_vector(31 downto 0);
			 float_op 		: out std_logic_VECTOR(5 downto 0);
			 float_add_res	: in std_logic_vector(31 downto 0);
			 float_mul_res	: in std_logic_vector(31 downto 0);
			 float_div_res	: in std_logic_vector(31 downto 0);
			 std_to_float 	: out std_logic_VECTOR(15 downto 0);
			 float_gen 		: in std_logic_VECTOR(31 downto 0);
			 float_to_std	: out std_logic_vector(31 downto 0);
			 std_gen			: in std_logic_VECTOR(15 downto 0);
			 Ahat_dto		: out std_logic_VECTOR(31 downto 0);
			 Ahat_addr		: out std_logic_VECTOR(5 downto 0); 
			 Ahat_we			: out std_logic_VECTOR(0 downto 0);
			 Ahat_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Bhat_dto		: out std_logic_VECTOR(31 downto 0);
			 Bhat_addr		: out std_logic_VECTOR(8 downto 0); 
			 Bhat_we			: out std_logic_VECTOR(0 downto 0);
			 Bhat_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Xdev_dto		: out std_logic_VECTOR(31 downto 0);
			 Xdev_addr		: out std_logic_VECTOR(4 downto 0); 
			 Xdev_we			: out std_logic_VECTOR(0 downto 0);
			 Xdev_dfrom		: in std_logic_VECTOR(31 downto 0);
			 J1_dto			: out std_logic_VECTOR(31 downto 0);
			 J1_addr			: out std_logic_VECTOR(9 downto 0); 
			 J1_we			: out std_logic_VECTOR(0 downto 0);
			 J1_dfrom		: in std_logic_VECTOR(31 downto 0);
			 G1_dto			: out std_logic_VECTOR(31 downto 0);
			 G1_addr			: out std_logic_VECTOR(5 downto 0); 
			 G1_we			: out std_logic_VECTOR(0 downto 0);
			 G1_dfrom		: in std_logic_VECTOR(31 downto 0);
			 Q_dto			: out std_logic_VECTOR(31 downto 0);
			 Q_addr			: out std_logic_VECTOR(7 downto 0); 
			 Q_we				: out std_logic_VECTOR(0 downto 0);
			 Q_dfrom			: in std_logic_VECTOR(31 downto 0);
			 Temp1_dto		: out std_logic_VECTOR(31 downto 0);
			 Temp1_addr		: out std_logic_VECTOR(8 downto 0); 
			 Temp1_we		: out std_logic_VECTOR(0 downto 0);
			 Temp1_dfrom	: in std_logic_VECTOR(31 downto 0);
			 C_dto			: out std_logic_VECTOR(31 downto 0);
			 C_addr			: out std_logic_VECTOR(3 downto 0); 
			 C_we				: out std_logic_VECTOR(0 downto 0);
			 C_dfrom			: in std_logic_VECTOR(31 downto 0);
			 T_dto			: out std_logic_VECTOR(31 downto 0);
			 T_addr			: out std_logic_VECTOR(5 downto 0); 
			 T_we				: out std_logic_VECTOR(0 downto 0);
			 T_dfrom			: in std_logic_VECTOR(31 downto 0);
			 L_dto			: out std_logic_VECTOR(31 downto 0);
			 L_addr			: out std_logic_VECTOR(5 downto 0); 
			 L_we				: out std_logic_VECTOR(0 downto 0);
			 L_dfrom			: in std_logic_VECTOR(31 downto 0);
			 G2_dto			: out std_logic_VECTOR(31 downto 0);
			 G2_addr			: out std_logic_VECTOR(5 downto 0); 
			 G2_we			: out std_logic_VECTOR(0 downto 0);
			 G2_dfrom		: in std_logic_VECTOR(31 downto 0)
		);
end MPC;

architecture basic of MPC is
type MPC_state_type is (init, create_obj, calc_min, getref, calc_output, output, delay);
signal MPC_state : MPC_state_type := init; -- MPC-regulator state machine

type init_state_type is (idle, delay, gen_ahat_1, gen_ahat_2, gen_bhat_1, gen_bhat_2, gen_bhat_3);	--MPC init states
signal init_state : init_state_type := gen_ahat_1;
signal init_nextstate : init_state_type := gen_ahat_1;

type obj_state_type is (delay, gen_xdev_1, gen_xdev_2, gen_J1_1, gen_J1_2, gen_G1_1, gen_G1_2, 
								gen_Q_1, gen_Q_2, gen_Q_3, gen_C_1);
signal obj_state : obj_state_type := gen_xdev_1;
signal obj_nextstate : obj_state_type := gen_xdev_1;

type calc_min_state_type is (delay, delay2, output, gen_G2_1);
signal calc_min_state : calc_min_state_type := gen_G2_1;
signal calc_min_nextstate : calc_min_state_type := gen_G2_1;


type MPC_array is array(0 to 9) of std_logic_vector(11 downto 0);	-- Defines an array of 10 "outputs"
signal MPC_u1_arr : MPC_array;												-- Creates such an array for "u1"
begin
	MPC_proc : process (clk) is
	variable delaycnt : integer := 0;
	constant timetodelay : integer := 200000;
	variable sampleno : integer := 0;
	variable g : integer := 0;
	variable h : integer := 0;
	variable i : integer := 0;
	variable j : integer := 0;
	variable k : integer := 0;
	variable l : integer := 0;
	variable delcnt : integer := 0;
	begin
	   if (clk='1' and clk'event) then
			case MPC_state is
				----------------------- Initiate all matrices needed for the QP-solver ---------------
				when init =>
					case init_state is
						when gen_ahat_1 =>	--Put A on top
							Ahat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 1);
							Ahat_dto <= Ad(i)(j); 
							Ahat_we <= "1";
							j := j+1;
							if(j=n_x) then
								j:=0;
								i := i+1;
								if(i=n_x) then
									i:=0;
									init_state <= gen_ahat_2;
									g := 0;h := 0;i := 0;j := 0;k := 1;
								end if;
							end if;
							init_nextstate <= gen_ahat_2;

						when gen_ahat_2 =>	--Generate A^2, A^3 etc downwards
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector(i+(k-1)*n_x, 5) & conv_std_logic_vector(h, 1);
									Ahat_we <= "0";
								when 1 => --We should multiply
									float_a <= Ahat_dfrom;
									float_b <= Ad(h)(j);
								when 2 => --Get value from Ahat where multiplication is "added to"
									Ahat_addr <= conv_std_logic_vector(i+k*n_x, 5) & conv_std_logic_vector(j, 1);
									Ahat_we <= "0";
								when 3 => --We should add
									float_a <= float_mul_res;
									float_b <= Ahat_dfrom;
								when 4 => --Write new value
									Ahat_addr <= conv_std_logic_vector(i+k*n_x, 5) & conv_std_logic_vector(j, 1);
									Ahat_dto <= float_add_res;
									Ahat_we <= "1";
								when others => g := 0;
							end case;

							init_nextstate <= gen_ahat_2;
							
							-- This makes this state into 5 nested for loops
							g := g+1;
							if(g=5) then
								g:=0;
								h := h+1;
								if(h=n_x) then
									h:=0;
									j := j+1;
									if(j=n_x) then
										j:=0;
										i:=i+1;
										if(i=n_x) then
											i:=0;
											k:=k+1;
											if(k=n_pred) then
												k := 0;
												init_nextstate <= gen_bhat_1;
											end if;
										end if;
									end if;
								end if;
							end if;
							
							init_state <= delay;
								
						when gen_bhat_1 =>	-- But Bd on top
							Bhat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 4);
							Bhat_dto <= Bd(i); 	-- Bd(i)(j) if several inputs
							Bhat_we <= "1";
							
							j := j+1;
							if(j=n_u) then
								j:=0;
								i := i+1;
								if(i=n_x) then
									i:=0;
									init_state <= gen_bhat_2;
									g := 0;h := 0;i := 0;j := 0;k := 1;
								end if;
							end if;
						
						when gen_bhat_2 =>	-- Complete column 1
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector((k-1)*n_x+i, 5) & conv_std_logic_vector(h, 1);
									Ahat_we <= "0";
								when 1 => --We should multiply
									float_a <= Ahat_dfrom;
									float_b <= Bd(h);		--Bd(h)(j) for several inputs
								when 2 => --Get value from Bhat where multiplication is "added to"
									Bhat_addr <= conv_std_logic_vector(k*n_x+i, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "0";
								when 3 => --We should add
									float_a <= float_mul_res;
									float_b <= Bhat_dfrom;
								when 4 => --Write new value
									Bhat_addr <= conv_std_logic_vector(k*n_x+i, 5) & conv_std_logic_vector(j, 4);
									Bhat_dto <= float_add_res;
									Bhat_we <= "1";
								when others => g := 0;
							end case;
						
							init_nextstate <= gen_bhat_2;
							
							-- This makes this state into 5 nested for loops
							g := g+1;
							if(g=5) then
								g:=0;
								h := h+1;
								if(h=n_x) then
									h:=0;
									j := j+1;
									if(j=n_u) then
										j:=0;
										i:=i+1;
										if(i=n_x) then
											i:=0;
											k:=k+1;
											if(k=n_pred) then
												k:=0;
												init_nextstate <= gen_bhat_3;
												g := 0;h := 0;i := n_x;j := n_u;k := 0;
											end if;
										end if;
									end if;
								end if;
							end if;
							
							init_state <= delay;
						
						when gen_bhat_3 => -- Copy column1 to the rest, shifted down one matrix							
							case g is
								when 0 => --Get value from Bhat
									Bhat_addr <= conv_std_logic_vector(i-n_x, 5) & conv_std_logic_vector(j-n_u, 4);
									Bhat_we <= "0";
								when 1 => --Output this to the correct position
									Bhat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "1";
									Bhat_dto <= Bhat_dfrom;
								when others => g := 0;
							end case;

							init_nextstate <= gen_bhat_3;
							g := g+1;
							if(g=2) then
								g:=0;
								i := i+1;
								if(i=n_x*n_pred) then
									i:=0;
									j := j+1;
									if(j=n_u*n_pred) then
										j:=0;
										init_nextstate <= idle;
										MPC_state <= create_obj;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							
							init_state <= delay;
							
						when idle => init_state <= idle;	
						when delay => init_state <= init_nextstate;
					end case;
				
				----------------------- Generate the objective function ---------------
				when create_obj =>
					case obj_state is
						when gen_xdev_1 =>	--Xdev=Ahat*x0;
							case g is
								when 0 => --Get value from Ahat
									Ahat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 1);
									Ahat_we <= "0";
									Xdev_addr <= conv_std_logic_vector(i, 5);
									Xdev_we <= "0";
								when 1 => --We should multiply Ahat(i,j)*x0(j)
									float_a <= Ahat_dfrom;
									float_b <= x0(j);
								when 2 => --We should add
									float_a <= float_mul_res;
									float_b <= Xdev_dfrom;
								when 3 => --Write new value
									Xdev_addr <= conv_std_logic_vector(i, 5);
									Xdev_dto <= float_add_res;
									Xdev_we <= "1";
								when others => g := 0;
							end case;
						
							obj_nextstate <= gen_xdev_1;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+1;
									if(i=n_x*n_pred) then
										i:=0;
										obj_nextstate <= gen_xdev_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_xdev_2 =>	--Xdev=Ahat*x0-xref;
							case g is
								when 0 => --Get value from Xdev
									Xdev_addr <= conv_std_logic_vector(i+j, 5);
									Xdev_we <= "0";
								when 1 => --We should subtract
									float_a <= Xdev_dfrom;
									float_b <= xref(j);
									float_op <= "000001";
								when 2 => --Write new value
									Xdev_addr <= conv_std_logic_vector(i+j, 5);
									Xdev_dto <= float_add_res;
									Xdev_we <= "1";
								when others => g := 0;
							end case;
							
							obj_nextstate <= gen_xdev_2;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+n_x;
									if(i=n_x*n_pred) then
										i:=0;
										obj_nextstate <= gen_J1_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_J1_1 =>
							case g is
								when 0 => --Insert -1 diagonally
									J1_addr <= conv_std_logic_vector(i, 6) & conv_std_logic_vector(i, 4);
									J1_dto <= "10111111100000000000000000000000"; -- -1
									J1_we <= "1";
								when 1 => --Insert 1 diagonally
									J1_addr <= conv_std_logic_vector(i+n_pred*n_u, 6) & conv_std_logic_vector(i, 4);
									J1_dto <= "00111111100000000000000000000000"; -- 1
									J1_we <= "1";
								when others => g := 0;
							end case;
							obj_nextstate <= gen_J1_1;
							g := g+1;
							if(g=2) then
								g:=0;
								i := i+1;
								if(i=n_pred*n_u) then
									i:=0;
									obj_nextstate <= gen_J1_2;
									g := 0;h := 0;i := 0;j := 0;k := 0;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_J1_2	=>				
							case g is
								when 0 => --Read from Bhat
									Bhat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "0";
								when 1 => --Insert into J1
									J1_addr <= conv_std_logic_vector(i+2*n_pred*n_u+n_pred*n_x, 6) & conv_std_logic_vector(j, 4);
									J1_dto <= Bhat_dfrom;
									J1_we <= "1";
									float_a <= Bhat_dfrom;
									float_b <= "10111111100000000000000000000000";	-- Multiply with -1
								when 2 => --Insert this one into J1 as well
									J1_addr <= conv_std_logic_vector(i+2*n_pred*n_u, 6) & conv_std_logic_vector(i, 4);
									J1_dto <= float_mul_res;
									J1_we <= "1";
								when others => g := 0;
							end case;
							obj_nextstate <= gen_J1_2;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_pred*n_u) then
									j:=0;
									i := i+1;
									if(i=n_pred*n_x) then
										obj_nextstate <= gen_G1_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_G1_1 =>
							case g is
								when 0 => --Get -umin
									float_a <= umin(j);
									float_b <= "10111111100000000000000000000000";	-- Multiply with -1
								when 1 => --Write to G1
									G1_addr <= conv_std_logic_vector(i+j, 6);
									G1_dto <= float_mul_res;
									G1_we <= "1";
								when 2 => --Insert into G1
									G1_addr <= conv_std_logic_vector(n_x*n_u+i+j, 6);
									G1_dto <= umax(j);
									G1_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_G1_1;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_u) then
									j:=0;
									i := i+n_u;
									if(i=n_u*n_pred) then
										obj_nextstate <= gen_G1_2;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_G1_2 =>
							case g is
								when 0 => --Get Xdev
									Xdev_addr <= conv_std_logic_vector(i+j, 5);
									Xdev_we <= "0";
									float_a <= xref(j);
									float_b <= xmin(j);
									float_op <= "000001";   -- Subtract
								when 1 => 
									float_a <= float_add_res;
									float_b <= Xdev_dfrom;
									float_op <= "000000";   -- Add
								when 2 => --Insert into G1
									G1_addr <= conv_std_logic_vector(i+j+2*N_x*n_u, 6);
									G1_dto <= float_add_res;
									G1_we <= "1";
								when 3 => 
									Xdev_addr <= conv_std_logic_vector(i+j, 5);
									Xdev_we <= "0";
									float_a <= xmax(j);
									float_b <= xref(j);
									float_op <= "000001";   -- Subtract
								when 4 => 
									float_a <= float_add_res;
									float_b <= Xdev_dfrom;
									float_op <= "000001";   -- Subtract again
								when 5 => --Insert into G1
									G1_addr <= conv_std_logic_vector(i+j+2*N_pred*n_u+N_pred*n_x, 6);
									G1_dto <= float_add_res;
									G1_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_G1_2;
							g := g+1;
							if(g=6) then
								g:=0;
								j := j+1;
								if(j=n_x) then
									j:=0;
									i := i+n_x;
									if(i=N_pred*n_x) then
										obj_nextstate <= gen_Q_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;

						when gen_Q_1 =>
							case g is
								when 0 => --Get Bhat
									Bhat_addr <= conv_std_logic_vector(i+k, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "0";
								when 1 => 
									float_a <= Bhat_dfrom;
									float_b <= Qx(k);
								when 2 =>
									temp1_addr <= conv_std_logic_vector(i+k, 5) & conv_std_logic_vector(j, 4);
									temp1_dto <= float_mul_res;
									temp1_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_Q_1;
							g := g+1;
							if(g=3) then
								g:=0;
								k := k+1;
								if(k=n_x) then
									k:=0;
									j := j+1;
									if(j=N_pred*n_u) then
										j:=0;
										i:=i+n_x;
										if(i=N_pred*n_x) then
											obj_nextstate <= gen_Q_2;
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end if;
							obj_state <= delay;
							
						when gen_Q_2 =>
							case g is
								when 0 => --Get Values
									Q_addr <= conv_std_logic_vector(i, 4) & conv_std_logic_vector(j, 4);
									Q_we <= "0";
									Bhat_addr <= conv_std_logic_vector(k, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "0";
									Temp1_addr <= conv_std_logic_vector(k, 5) & conv_std_logic_vector(i, 4);
									Temp1_we <= "0";
								when 1 => 
									float_a <= Bhat_dfrom;
									float_b <= Temp1_dfrom;
								when 2 =>
									float_a <= float_mul_res;
									float_b <= Q_dfrom;
									float_op <= "000000";
								when 3 =>
									Q_addr <= conv_std_logic_vector(i, 4) & conv_std_logic_vector(j, 4);
									Q_dto <= float_add_res;
									Q_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_Q_2;
							g := g+1;
							if(g=4) then
								g:=0;
								k := k+1;
								if(k=n_x*n_pred) then
									k:=0;
									j := j+1;
									if(j=N_pred*n_u) then
										j:=0;
										i:=i+i;
										if(i=N_pred*n_u) then
											obj_nextstate <= gen_Q_3;
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_Q_3 =>
							case g is
								when 0 => --Get Values
									Q_addr <= conv_std_logic_vector(i+j, 4) & conv_std_logic_vector(i+j, 4);
									Q_we <= "0";
								when 1 => 
									float_a <= Q_dfrom;
									float_b <= Qu(j);
									float_op <= "000000";
								when 2 =>
									Q_addr <= conv_std_logic_vector(i+j, 4) & conv_std_logic_vector(i+j, 4);
									Q_dto <= float_add_res;
									Q_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_Q_3;
							g := g+1;
							if(g=3) then
								g:=0;
								j := j+1;
								if(j=n_u) then
									j:=0;
									i := i+n_u;
									if(i=N_pred*n_u) then
										obj_nextstate <= gen_C_1;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when gen_C_1 =>
							case g is
								when 0 => --Get Values
									C_addr <= conv_std_logic_vector(i, 4);
									C_we <= "0";
									Xdev_addr <= conv_std_logic_vector(j, 5);
									Xdev_we <= "0";
									Temp1_addr <= conv_std_logic_vector(j, 5) & conv_std_logic_vector(i, 4);
									Temp1_we <= "0";
								when 1 => 
									float_a <= Xdev_dfrom;
									float_b <= Temp1_dfrom;
								when 2 => 
									float_a <= float_mul_res;
									float_b <= C_dfrom;
									float_op <= "000000";
								when 3 =>
									C_addr <= conv_std_logic_vector(i, 4);
									C_dto <= float_add_res;
									C_we <= "1";
								when others => g:=0;
							end case;
							obj_nextstate <= gen_C_1;
							g := g+1;
							if(g=4) then
								g:=0;
								j := j+1;
								if(j=n_x * n_pred) then
									j:=0;
									i := i+1;
									if(i=N_pred*n_u) then
										obj_nextstate <= gen_xdev_1;
										MPC_state <= calc_min;
										g := 0;h := 0;i := 0;j := 0;k := 0;
									end if;
								end if;
							end if;
							obj_state <= delay;
						
						when delay => obj_state <= obj_nextstate;
					end case; -- Finished generating object function

				----------------------- Find the minimizing vector ---------------
				when calc_min =>
					case calc_min_state is
						when gen_G2_1 =>	--G2(i)=-t(i)/l(i);
							case g is
								when 0 =>
									T_addr <= conv_std_logic_vector(i, 6);
									T_we <= "0";
									L_addr <= conv_std_logic_vector(i, 6);
									L_we <= "0";
								when 1 =>	--Divide these
									float_a <= T_dfrom;
									float_b <= L_dfrom;
								when 2 =>	--Insert -(result)
									G2_addr <= conv_std_logic_vector(i, 6);
									G2_dto(30 downto 0) <= float_div_res(30 downto 0);
									G2_dto(31) <= not(float_div_res(31));
									G2_we <= "1";
								when others => g:=0;
							end case;
							
							calc_min_state <= output;
							
						when output =>
							case g is
								when 0 =>
									Bhat_addr <= conv_std_logic_vector(i, 5) & conv_std_logic_vector(j, 4);
									Bhat_we <= "0";
								when 1 =>
									debug <= Bhat_dfrom;
								when others => g := 0;
								calc_min_state <= delay;
								g := g+1;
								if(g=2) then
									g:=0;
									calc_min_state <= delay2;
									j := j+1;
									if(j=n_u * n_pred) then
										j:=0;
										i := i+1;
										if(i=n_x * n_pred) then
											g := 0;h := 0;i := 0;j := 0;k := 0;
										end if;
									end if;
								end if;
							end case;
							
						when delay => calc_min_state <= calc_min_nextstate;
						when delay2 =>
							delcnt := delcnt +1;
							if(delcnt = 50000000) then
									calc_min_state <= calc_min_nextstate;
									delcnt := 0;
							end if;
					end case; -- Finished calculating the minimizer
				
				
				when getref => -- Generate reference matrix
				when calc_output => --calculate output for the next ten samples
				
					delaycnt := delaycnt + 1;
					if(delaycnt = timetodelay) then
						MPC_u1 <= MPC_u1_arr(0);
						delaycnt := 0;
					end if;
				when others => MPC_state <= init;
			end case;
		end if;
	end process MPC_proc;
end architecture basic;