--PID block-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity model is
   Port ( clk 				: in std_logic;
			 SYS_state 		: in SYS_state_type;
			 Out_Value 		: in std_logic_vector(11 downto 0);
			 x0 				: out array1;
			 float_a 		: out std_logic_vector(31 downto 0);
			 float_b 		: out std_logic_vector(31 downto 0);
			 float_op 		: out std_logic_VECTOR(5 downto 0);
			 float_add_res	: in std_logic_vector(31 downto 0);
			 float_mul_res	: in std_logic_vector(31 downto 0);
			 float_div_res	: in std_logic_vector(31 downto 0);
			 std_to_float 	: out std_logic_VECTOR(15 downto 0);
			 float_gen 		: in std_logic_VECTOR(31 downto 0);
			 float_to_std	: out std_logic_vector(31 downto 0);
			 std_gen			: in std_logic_VECTOR(15 downto 0)
   );
end model;

architecture basic of model is
type MODEL_state_type is (idle, calc_v, calc_v_2, calc_states_1, calc_states_2, calc_states_3, calc_states_4, 
								  calc_states_5, calc_states_6, calc_states_7, calc_states_8, calc_states_9, 
								  output_1, output_2, output_3, delay);
signal MODEL_state : MODEL_state_type := idle; -- Statemachine for modelling process
signal MODEL_nextstate : MODEL_state_type := idle; -- Statemachine for modelling process
begin
	model_proc : process (clk) is
	variable v : std_logic_vector(31 downto 0);															-- applied voltage
	
	variable x : array1 		:= ("00000000000000000000000000000000","00000000000000000000000000000000");
	variable x_int : array1 := ("00000000000000000000000000000000","00000000000000000000000000000000");

	variable i : integer range 0 to n_x := 0;
	variable j : integer range 0 to n_x := 0;
	
	variable ready_to_run : bit := '0';
	
	begin
	   if (clk='1' and clk'event) then
			case MODEL_state is
				when idle   => if((SYS_state = run_SPI) and (ready_to_run = '1')) then 
										MODEL_state <= calc_v; ready_to_run := '0';
									end if;
									if(SYS_state /=  run_SPI) then ready_to_run := '1'; end if;
				when calc_v => std_to_float <= "00000000" & Out_Value(11 downto 4); --Divide by 16, as 4095 = 256 V
									MODEL_nextstate <= calc_v_2; 
									MODEL_state <= delay;
								
				when calc_v_2 => v := float_gen;
									MODEL_state <= calc_states_1;
						
				-- This adds A*x to the integrator matrix		
				when calc_states_1 =>
									float_a <= A(i)(j);
									float_b <= x(j);
									
									MODEL_nextstate <= calc_states_2; 
									MODEL_state <= delay;
									
				when calc_states_2 =>
									float_a <= float_mul_res;
									float_b <= x_int(i);
									float_op <= "000000";
									
									MODEL_nextstate <= calc_states_3; 
									MODEL_state <= delay;
							
				when calc_states_3 => 
									x_int(i) := float_add_res;
									if(j=n_x-1) then 
										j := 0;
										if(i=n_x-1) then
											i := 0;
											MODEL_state <= calc_states_4;
										else 
											i := i+1;
											MODEL_state <= calc_states_1;
										end if;
									else 
										j := j+1;
										MODEL_state <= calc_states_1;
									end if;
					
				-- This adds B*u to the x_dot matrix		
				when calc_states_4 => 
									float_a <= B(i);
									float_b <= v;
									
									MODEL_nextstate <= calc_states_5; 
									MODEL_state <= delay;	

				when calc_states_5 =>
									float_a <= float_mul_res;
									float_b <= x_int(i);
									float_op <= "000000";
									
									MODEL_nextstate <= calc_states_6; 
									MODEL_state <= delay;

				when calc_states_6 => 
									x_int(i) := float_add_res;
									if(i=n_x-1) then
										i := 0;
										MODEL_state <= calc_states_7;
									else 
										i := i+1;
										MODEL_state <= calc_states_4;
									end if;							
				
				-- This converts the integrators to correct values (divides by sample time)
				when calc_states_7 => 
									float_a <= x_int(i);
									float_b <= sample_time_float;
											
									MODEL_nextstate <= calc_states_8; 
									MODEL_state <= delay;
								
				when calc_states_8 => 
									x(i) := float_div_res;
									
									if(i=n_x-1) then
										i := 0;
										MODEL_state <= output_1;
									else 
										i := i+1;
										MODEL_state <= calc_states_7;
									end if;		
				
				-- Convert the outvalue to correct form
				when output_1 =>
									float_a <= x(1);
									float_b <= "01000001100000000000000000000000";	--mul by 16 to convert rad/s to invalue
									
									MODEL_nextstate <= output_2; 
									MODEL_state <= delay;

				when output_2 => float_to_std <= float_mul_res;	
									MODEL_nextstate <= output_3;
									MODEL_state <= delay;
									
				when output_3 => P_Value_mod <= std_gen(11 downto 0);
									--x0(0) <= x(0);
									--x0(1) <= x(1);
									x0(0) <= "01100001111001001111100";
									x0(1) <= "01100001111001001111100";
									MODEL_state <= idle;

				when delay => 	MODEL_state <= MODEL_nextstate;
				when others => 	MODEL_state <= idle;
			end case;
			
		end if;
	end process model_proc;
	
end architecture basic;