--UART--------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;
use work.mpc_package.all;

entity uart is
   Port ( UART_CLK 	: in std_logic;
			 UART_RXD 	: in std_logic;
			 UART_TXD 	: out std_logic := '0';
			 UART_DATA 	: inout UART_queue;
			 Xref 		: out array1 := ("00000000000000000000000000000000",	--x0 ref = 0
												  "00000000000000000000000000000000");	  	--x1 ref = 0
			 debug		: in std_logic_vector(31 downto 0)
   );
end uart;

architecture basic of uart is
signal outbyte : std_logic_vector(7 downto 0) := "00000000";
signal send : std_logic := '0';
begin
	read_proc : process (UART_CLK)
	variable inbit 	: integer range 0 to 10 := 0;
	variable inbyte 	: std_logic_vector(7 downto 0) := "00000000";
	variable incoming : std_logic := '1';
	variable charno 	: integer range 0 to 5 := 0;
	variable newcmd 	: std_logic := '0';
	begin
	   if (UART_CLK='1' and UART_CLK'event) then
			if(send='1') then send <= '0'; end if;
			if(incoming = '1') then incoming := UART_RXD; end if;
			if((incoming = '0') and (newcmd='0')) then
				case inbit is
					when 0 => 			inbit := inbit + 1;			
					when 1|2|3|4|5|6|7|8 => 
											inbyte(inbit - 1) := UART_RXD; 	--Data
											inbit := inbit + 1;
					when 9  => 			inbit := inbit + 1;
					when 10 => 			UART_DATA(charno) <= inbyte;
											--outbyte <= inbyte;
											send <= '1';
											if(charno < 5 and inbyte /= "00001000") then charno := charno + 1; end if;	--Get ready for next char
											if(inbyte = "00001101") then  							--Carriage return
												charno := 0; 
												newcmd := '1'; 
											end if; 
											if((inbyte = "00001000") and (charno > 0)) then  	--Backspace
												UART_DATA(charno) <= "00000000";
												UART_DATA(charno-1) <= "00000000";
												charno := charno-1; 
											end if; 	
											inbit := 0;
											incoming := '1';		
				end case;
			end if;
			
			if(newcmd='1') then									-- Try to read command from UART-data
				if(UART_DATA(0)="00110000") then
					Xref  <= ("00000000000000000000000000000000",
								"00000000000000000000000000000000");	--0
				elsif(UART_DATA(0)="00110001") then
					Xref  <= ("00000000000000000000000000000000",
								"01000010010010000000000000000000");	--50
				elsif(UART_DATA(0)="00110010") then
					Xref  <= ("00000000000000000000000000000000",
								"01000010110010000000000000000000");	--100
				elsif(UART_DATA(0)="00110011") then
					Xref  <= ("00000000000000000000000000000000",
								"01000011000101100000000000000000");	--150
				elsif(UART_DATA(0)="00110100") then
					Xref  <= ("00000000000000000000000000000000",
								"01000011010010000000000000000000");	--200
				elsif(UART_DATA(0)="00110101") then
					Xref  <= ("00000000000000000000000000000000",
								"01000011011110100000000000000000");	--250
				elsif(UART_DATA(0)="00110110") then
					Xref  <= ("00000000000000000000000000000000",
								"01000011100101100000000000000000");	--300
				elsif(UART_DATA(0)="00110111") then
					Xref  <= ("00000000000000000000000000000000",
								"01000011101011110000000000000000");	--350
				elsif(UART_DATA(0)="00111111") then
					Xref  <= ("11111111111111111111111111111111",
								"11111111111111111111111111111111");	--0	
				end if;
				newcmd := '0';
			end if; -- End if newcmd.
		end if;	-- End clk trigger.
	end process read_proc;
	
	write_proc : process (UART_CLK)
	variable outbit : integer range 0 to 10 := 0;
	variable int_send : std_logic := '0';
	variable count : integer range 0 to 33 := 33;
	variable timecnt : integer := 0;
	begin
	   if (UART_CLK='1' and UART_CLK'event) then
			timecnt := timecnt + 1;
			if(timecnt = 9600) then int_send := '1'; timecnt := 0; end if;
			if(int_send = '1') then
				if(count > 1) then outbyte <= "0011000" & debug(count-2);	--Send data
				elsif(count=1) then outbyte <= "00001010";	--Linefeed
				else outbyte <= "00001101";	--Carriage return
				end if;
				case outbit is
					when 0 => 	UART_TXD <= '1';			--Startbit
									outbit := outbit + 1;
					when 1 => 	UART_TXD <= '0';			--Startbit
									outbit := outbit + 1;				
					when 2|3|4|5|6|7|8|9 => 
									UART_TXD <= outbyte(outbit - 2); 	--Data
									outbit := outbit + 1;
					when 10 => 	UART_TXD <= '0';			--Stopbit 1
									outbit := 0;
									if(count=0) then count := 33; int_send := '0';
									else count := count - 1;
									end if;
				end case;
			end if;
		end if;
	end process write_proc;
end architecture basic;