--PID block-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity PID is
   Port ( clk : in std_logic;
			 P_Value : in std_logic_vector(11 downto 0);
			 Ref : in std_logic_vector(7 downto 0);
			 SYS_state : in SYS_state_type;
			 PID_done : inout std_logic := '0'; 
			 Cont_Value : inout std_logic_vector(11 downto 0) := "000000000000";
			 Kp : in std_logic_vector(7 downto 0);
			 Ti : in std_logic_vector(7 downto 0);
			 Td : in std_logic_vector(3 downto 0)
   );
end PID;

architecture basic of PID is
type PID_state_type is (calc_error, calc_PID, calc_sum);
signal PID_state : PID_state_type := calc_error; -- transmitter's SPI_state
begin
	PID_proc : process (clk) is
	variable error : integer := 0; 									--Reference - P_Values
	variable sum : integer := 0;
	--Proportional gain
	variable PropOut : integer;											--Proportional part
	--Integrator
	variable Accum_error : integer := 0;
	variable real_error : integer := 0;								--Accum_error divided by sample time
	variable integrator_temp : integer := 0;
	variable Anti_Windup : integer range -4095 to 4095;		--Anti windup value
	variable IntegratorOut : integer range -4095 to 4095;	--Integrator part
	--Derivator
	variable DerivatorOut : integer;									--Derivator part
	variable error_t1 : integer := 0;
	begin
	   if (clk='1' and clk'event) then
			if(SYS_state = calc_PID and PID_done = '0') then
				case PID_state is
				when calc_error => 	error := (conv_integer(Ref)*16)-conv_integer(P_Value);		--	Calc error
											PID_state <= calc_PID;
				when calc_PID => 		Propout := error*conv_integer(Kp(7 downto 4)) + (error*conv_integer(Kp(3 downto 0)))/8 - (error*conv_integer(Kp(3 downto 0)))/32;
				
											if((Cont_Value < 4095) and (error > 0)) then 	-- Calc I
												Accum_error := Accum_error + error;
											elsif((Cont_Value > 0) and (error < 0)) then
												Accum_error := Accum_error + error;
											end if;
											real_error := Accum_error / sample_time;
											integrator_temp := (conv_integer(Ti(7 downto 4))*real_error) + (conv_integer(Ti(3 downto 0))*real_error)/8 - (conv_integer(Ti(3 downto 0))*real_error)/32;
											-- Apply integrator saturation. Is this really needed when we have Anti-Windup??
											if(integrator_temp > 4095) then
												integrator_temp := 4095;
											end if;
											if(integrator_temp < -4095) then
												integrator_temp := -4095;
											end if;
											-- Output new value
											IntegratorOut := integrator_temp;
								
											DerivatorOut := conv_integer(Td) * (error - error_t1);			--Calc D
											error_t1 := error;
											
											PID_state <= calc_sum;
								
				when calc_sum => 	sum := PropOut + IntegratorOut;			--Sum the parts, create output value
								if(sum > 4095) then
								sum := 4095;
								end if;
								if(sum < 0) then
									sum := 0;
								end if;
								Cont_Value <= conv_std_logic_vector(sum, 12); 		-- Control-value is 0-4095, or 12 bits	
								
								PID_done <= '1';
								PID_state <= calc_error;
				end case;
			elsif(SYS_state = idle) then PID_done <= '0';
			end if;
		end if;
	end process PID_proc;
	
end architecture basic;