----------------------------------------------------------------------------------
-- Company: Siemens Oil & Gas
-- Engineer: Erik Nss
-- 
-- Create Date:    23:06:03 01/31/2009 
-- Design Name:    PID Regulator
-- Module Name:    PID_Reg - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Masters assignment,  under supervision by Karstein Kristiansen (Siemens) &
-- 				 Sverre Hendseth (NTNU - Teknisk Kybernetikk)
--
-- Dependencies: ChipScope must probably be installed to compile properly. This can be downloaded
--					  for free from Xilinx's webpages.
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--FULL REGULATOR------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity PID_Regulator is
   Port ( clk : in std_logic;
			 --SPI
			 SPI_MISO : in std_logic;
			 SPI_MOSI : out std_logic;
			 SPI_SCK : inout std_logic;
			 --UART
			 UART_RXD : in std_logic;
			 UART_TXD : out std_logic;
			 --ADC and Ampifier
			 AD_CONV : out std_logic;
			 AMP_CS : out std_logic;
			 AMP_SHDN : out std_logic;
			 --DAC
			 DAC_CS : out std_logic;
			 DAC_CLR : out std_logic;
			 --LCD-interface
			 SF_D_11,SF_D_10,SF_D_9,SF_D_8 : out std_logic;
			 LCD_E,LCD_RS,LCD_RW : out std_logic;
			 --LEDS
			 LED_0, LED_1, LED_2, LED_3, LED_4, LED_5, LED_6, LED_7 : out std_logic;
			 --VGA
			 r, g, b, hsync, vsync : out std_logic;
			 --BUTTONS
			 ROT_CENTER : in std_logic;
			 ROT_A : in std_logic := '0';
			 ROT_B : in std_logic := '0';
			 BTN_NORTH : in std_logic := '0';
			 BTN_WEST : in std_logic := '0';
			 BTN_SOUTH : in std_logic := '0';
			 BTN_EAST : in std_logic := '0';
			 --SPI-Devices to be disabled
			 SPI_SS_B : out std_logic;
			 SF_CE0 : out std_logic;
			 FPGA_INIT_B : out std_logic
   );
end PID_Regulator;

architecture struct of PID_Regulator is
	--Internal signals
	--VGA signals
	signal VGA_CLK : std_logic;									--This is the 25 MHz VGA clock.
	signal red, green, blue : std_logic;
	signal row : std_logic_vector(8 downto 0);
	signal column : std_logic_vector(9 downto 0);
	--RAM signals
	signal RAM_dto: std_logic_VECTOR(31 downto 0);
	signal RAM_addr: std_logic_VECTOR(11 downto 0);
	signal RAM_we: std_logic_VECTOR(0 downto 0);
	signal RAM_dfrom: std_logic_VECTOR(31 downto 0);
	--UART
	signal UART_CLK : std_logic;	
	signal UART_DATA : UART_queue;
	--PID regulator
	signal Kp : std_logic_vector(7 downto 0);					--Floating point on form XXXX.XXXX
	signal Ti : std_logic_vector(7 downto 0);
	signal Td : std_logic_vector(3 downto 0);
	
	--Arithmetics
	--Used for model
	signal float_a : std_logic_VECTOR(31 downto 0);
	signal float_b : std_logic_VECTOR(31 downto 0);
	signal float_op : std_logic_VECTOR(5 downto 0);
	signal float_add_res : std_logic_VECTOR(31 downto 0);
	signal float_mul_res : std_logic_VECTOR(31 downto 0);
	signal float_div_res : std_logic_VECTOR(31 downto 0);
   signal std_to_float : std_logic_VECTOR(15 downto 0);
	signal float_gen : std_logic_VECTOR(31 downto 0);
	signal float_to_std : std_logic_VECTOR(31 downto 0);
	signal std_gen : std_logic_VECTOR(15 downto 0);
	
	--other
	signal reset : std_logic;										--Resets the system. Using BTN_EAST.
	signal P_Value : std_logic_vector(11 downto 0);			--Process Value read from ADC: 0 to 4095
	signal P_Value_mod : std_logic_vector(11 downto 0);	--Process Value from model: 0 to 4095
	signal Ref : std_logic_vector(7 downto 0);				--Reference
	signal debug : std_logic_vector(31 downto 0); 			--Used for debugging.
	signal Out_Value : std_logic_vector(11 downto 0);		--Value to sent to DAC/process ("paadrag")
	signal PID_u1 : std_logic_vector(11 downto 0);			--Outvalue generated by PID-controller
	signal SYS_state : SYS_state_type; 							--Global statemachine
	signal SPI_done, PID_done : std_logic;						--Signals for trigging the statemachine
	signal mode : std_logic_vector(1 downto 0);				--00 for manual, 01 for PID, 10 for MPC
begin
	--Clock generator
	b_clk_gen : entity work.clk_gen(basic)
		port map (clk, SPI_SCK, VGA_CLK, UART_CLK);
	--UART
	b_UART : entity work.uart(basic)
		port map (UART_CLK, UART_RXD, UART_TXD, UART_DATA, Kp, Ti, Td, mode, debug, Ref);
	--Other
	--b_ref_gen : entity work.ref_gen(basic)
	--   port map (clk, ROT_A, ROT_B, Ref);
	b_spi : entity work.spi_interface(basic)
	   port map (SPI_SCK, SPI_MISO, SPI_MOSI, AMP_CS, AMP_SHDN, SYS_state,
				    AD_CONV, P_Value, DAC_CS, DAC_CLR, Out_Value, SPI_done);
	--Regulator
	b_PID : entity work.PID(basic)
		port map (clk, P_Value, Ref, SYS_state, PID_done, PID_u1, Kp, Ti, Td);
	--Auto/manual selector
	b_automan : entity work.automan(basic)
		port map (clk, PID_u1, Ref, mode, Out_Value);
	--Model for MPC
	--b_model : entity work.model(basic)
		--port map (clk, SYS_state, Out_Value, P_Value_mod, float_a, float_b, float_op, float_add_res, float_mul_res, float_div_res, std_to_float, float_gen, float_to_std, std_gen);
	--LED and LCD controller
	--b_LED_CTRL : entity work.leds(basic)
		--port map (clk, LED_0, LED_1, LED_2, LED_3, LED_4, LED_5, LED_6, LED_7, Ref);
	--b_LCD_CTRL : entity work.lcd(basic)
		--port map (clk, reset, SF_D_11, SF_D_10, SF_D_9, SF_D_8, LCD_E, LCD_RS, LCD_RW, P_Value, Out_Value);
	--VGA
	--b_vga : entity work.vga(basic)
	   --port map (VGA_CLK, red, green, blue, r, g, b, hsync, vsync, row, column);
	--b_screen_gen : entity work.screen_gen(basic)
		--port map (VGA_CLK, row, column, red, green, blue, Ref, mode, P_Value, P_Value_mod, Out_Value, RAM_dto, RAM_addr, RAM_we, RAM_dfrom, BTN_NORTH, BTN_SOUTH, ROT_CENTER, UART_DATA, Kp, Ti, Td);
	--Other
	b_SYS_SM : entity work.SYS_SM(basic)
		port map (clk, reset, SYS_state, SPI_done, PID_done);
	b_GROUND : entity work.ground(basic)
		port map (clk,b,g,r,hsync,vsync,
					 LED_0,LED_1,LED_2,LED_3,LED_4,LED_5,LED_6,LED_7,
					 SF_D_8,SF_D_9,SF_D_10,SF_D_11,LCD_E,LCD_RS,LCD_RW,
					 SPI_SS_B, SF_CE0, FPGA_INIT_B);
		
	------------------- COREGEN BLOCKS ------------------------
	--RAM storing process, input, model and reference history.
	--b_RAM : entity work.RAM(RAM_a)
		--port map (clk, RAM_dto, RAM_addr, RAM_we, RAM_dfrom);

	--Mathematical functions (float operations)
	--b_float_add : entity work.float_add(float_add_a)
		--port map (float_a, float_b, float_op, clk, float_add_res);
	--b_float_mul : entity work.float_mul(float_mul_a)
		--port map (float_a, float_b, clk, float_mul_res);
	--b_float_div : entity work.float_div(float_div_a)
		--port map (float_a, float_b, clk, float_div_res);
	--b_float_gen : entity work.float_gen(basic)							-- std_logic_vec to float
		--port map (clk, std_to_float, float_gen);
	--b_std_gen : entity work.std_gen(basic)									-- float to std_logic_vec
		--port map (clk, float_to_std, std_gen);
	
end architecture struct;