--PID block-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity model is
   Port ( clk 				: in std_logic;
			 SYS_state 		: in SYS_state_type;
			 Out_Value 		: in std_logic_vector(11 downto 0);
			 P_Value_mod 	: out std_logic_vector(11 downto 0) := "000000000000";
			 float_a 		: out std_logic_vector(31 downto 0);
			 float_b 		: out std_logic_vector(31 downto 0);
			 float_op 		: out std_logic_VECTOR(5 downto 0);
			 float_add_res	: in std_logic_vector(31 downto 0);
			 float_mul_res	: in std_logic_vector(31 downto 0);
			 float_div_res	: in std_logic_vector(31 downto 0);
			 std_to_float 	: out std_logic_VECTOR(15 downto 0);
			 float_gen 		: in std_logic_VECTOR(31 downto 0);
			 float_to_std	: out std_logic_vector(31 downto 0);
			 std_gen			: in std_logic_VECTOR(15 downto 0);
			 debug			: out std_logic_VECTOR(31 downto 0)
   );
end model;

architecture basic of model is
type MODEL_state_type is (idle, calc_v, calc_x1_1, calc_x1_2, calc_x1_3, calc_x1_4, calc_x1_5, calc_x1_6, 
									calc_x2_1, calc_x2_2, calc_x2_3, calc_x2_4, calc_x2_5, calc_x2_6, calc_x2_7, 
									calc_x1, calc_x2, output_1, output_2, delay);
signal MODEL_state : MODEL_state_type := idle; -- Statemachine for modelling process
signal MODEL_nextstate : MODEL_state_type := idle; -- Statemachine for modelling process
begin
	model_proc : process (clk) is
	variable v : std_logic_vector(31 downto 0);															-- applied voltage
	
	variable x1_integrator : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	variable x1 : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";		-- motor current
	variable x2_integrator : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	variable x2 : std_logic_vector(31 downto 0) := "00000000000000000000000000000000";		-- motor speed, radians/sec
	
	variable ready_to_run : bit := '0';
	
	begin
	   if (clk='1' and clk'event) then
			case MODEL_state is
				when idle   => if((SYS_state = run_SPI) and (ready_to_run = '1')) then 
										MODEL_state <= calc_v; ready_to_run := '0';
									end if;
									if(SYS_state /=  run_SPI) then ready_to_run := '1'; end if;
				when calc_v => std_to_float <= "00000000" & Out_Value(11 downto 4); --Divide by 16, as 4095 = 256 V
									MODEL_nextstate <= calc_x1_1; 
									MODEL_state <= delay;
									
				when calc_x1_1 => v := float_gen;									-- got v (input voltage)
									float_a <= A(0)(0);
									float_b <= x1;
									
									MODEL_nextstate <= calc_x1_2; 
									MODEL_state <= delay;
									
				when calc_x1_2 =>
									float_a <= float_mul_res;
									float_b <= x1_integrator;
									float_op <= "000000";						-- perform addition
									
									MODEL_nextstate <= calc_x1_3; 
									MODEL_state <= delay;
									
				when calc_x1_3 => x1_integrator := float_add_res;
									float_a <= A(0)(1);
									float_b <= x2;
									
									MODEL_nextstate <= calc_x1_4; 
									MODEL_state <= delay;
									
				when calc_x1_4 =>
									float_a <= float_mul_res;
									float_b <= x1_integrator;
									float_op <= "000000";						-- perform addition
									
									MODEL_nextstate <= calc_x1_5; 
									MODEL_state <= delay;
									
				when calc_x1_5 => x1_integrator := float_add_res;
									float_a <= B1;
									float_b <= v;
									
									MODEL_nextstate <= calc_x1_6; 
									MODEL_state <= delay;
									
				when calc_x1_6 =>
									float_a <= float_mul_res;
									float_b <= x1_integrator;
									float_op <= "000000";						-- perform addition
									
									MODEL_nextstate <= calc_x2_1; 
									MODEL_state <= delay;
									
				when calc_x2_1 => x1_integrator := float_add_res;
									float_a <= A(1)(0);
									float_b <= x1;
									
									MODEL_nextstate <= calc_x2_2; 
									MODEL_state <= delay;
				
				when calc_x2_2 =>
									float_a <= float_mul_res;
									float_b <= x2_integrator;
									float_op <= "000000";	
									
									MODEL_nextstate <= calc_x2_3; 
									MODEL_state <= delay;
								
				when calc_x2_3 => x2_integrator := float_add_res;
									float_a <= A(1)(1);
									float_b <= x2;
									
									MODEL_nextstate <= calc_x2_4; 
									MODEL_state <= delay;
								
				when calc_x2_4 =>
									float_a <= float_mul_res;
									float_b <= x2_integrator;
									float_op <= "000000";	
									
									MODEL_nextstate <= calc_x2_5; 
									MODEL_state <= delay;
									
				when calc_x2_5 => x2_integrator := float_add_res;
									float_a <= B2;
									float_b <= v;
									
									MODEL_nextstate <= calc_x2_6; 
									MODEL_state <= delay;
									
				when calc_x2_6 =>
									float_a <= float_mul_res;
									float_b <= x2_integrator;
									float_op <= "000000";
									
									MODEL_nextstate <= calc_x2_7; 
									MODEL_state <= delay;					
				
				when calc_x2_7 => x2_integrator := float_add_res;
									float_a <= x1_integrator;
									float_b <= sample_time_float;	
									
									MODEL_nextstate <= calc_x1; 
									MODEL_state <= delay;
									
				when calc_x1 => x1 := float_div_res;						-- increased w-integrator
									float_a <= x2_integrator;
									float_b <= sample_time_float;				--div by sample time
									
									MODEL_nextstate <= calc_x2; 
									MODEL_state <= delay;
									
				when calc_x2 => x2 := float_div_res;						-- increased w-integrator
									float_a <= x2;
									float_b <= "01000001100000000000000000000000";	--mul by 16 to convert rad/s to invalue
									
									MODEL_nextstate <= output_1; 
									MODEL_state <= delay;

				when output_1 => float_to_std <= float_mul_res;
									
									MODEL_nextstate <= output_2;
									MODEL_state <= delay;
									
				when output_2 => P_Value_mod <= std_gen(11 downto 0);
									
										MODEL_state <= idle;

				when delay => 	MODEL_state <= MODEL_nextstate;
				when others => 	MODEL_state <= idle;
			end case;
			
		end if;
	end process model_proc;
	
end architecture basic;