----------------------------- CHAR GENERATOR ------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity SCREEN_GEN is
	Port (VGA_CLK : in std_logic;
			row : in std_logic_vector(8 downto 0);
			column : in std_logic_vector(9 downto 0);
			red, green, blue : out std_logic;
			Ref : in std_logic_vector(7 downto 0);
			mode : in std_logic_vector(1 downto 0);
			P_Value : in std_logic_vector(11 downto 0);
			P_Value_mod : in std_logic_vector(11 downto 0);
			Out_Value : in std_logic_vector(11 downto 0);
			RAM_dto: out std_logic_VECTOR(31 downto 0) := "00000000000000000000000000000000";
			RAM_addr: out std_logic_VECTOR(11 downto 0) := "000000000000";
			RAM_we: out std_logic_VECTOR(0 downto 0) := "0";
			RAM_dfrom: in std_logic_VECTOR(31 downto 0);
			BTN_NORTH : in std_logic;
			BTN_SOUTH : in std_logic;
			ROT_CENTER : in std_logic;
			UART_DATA : in UART_queue;
			Kp				: in std_logic_vector(7 downto 0);
			Ti				: in std_logic_vector(7 downto 0);
			Td				: in std_logic_vector(3 downto 0)
   );
end SCREEN_GEN;

architecture basic of SCREEN_GEN is
signal shiftfreq : integer := 1953125;
signal speed : integer range 1 to 5 := 5;
signal freezeimg : bit := '0';
signal char : std_logic_vector(7 downto 0) := "00000000";
signal vec_0, vec_1, vec_2, vec_3, vec_4, vec_5, vec_6, vec_7 : std_logic_vector(7 downto 0);
begin
	b_char_gen : entity work.char_gen(basic)
		port map (VGA_CLK, char, vec_0, vec_1, vec_2, vec_3, vec_4, vec_5, vec_6, vec_7);
		
	btn_debouce : process (VGA_CLK)
	variable debounce_cnt : integer := 0;
	begin
	if (VGA_CLK'event and VGA_CLK='1') then
		if(debounce_cnt > 0) then debounce_cnt:=debounce_cnt-1; end if;
		if((BTN_NORTH='1') and (debounce_cnt=0) and (speed<5)) then
			speed<=speed+1;
			debounce_cnt := 10000000; --Wait for 10000000 cycles, or 200 ms before allowing new speed change
		end if;
		if((BTN_SOUTH='1') and (debounce_cnt=0) and (speed>1)) then
			speed<=speed-1;
			debounce_cnt := 10000000; --Wait for 10000000 cycles, or 200 ms before allowing new speed change
		end if;
		if((ROT_CENTER='1') and (debounce_cnt=0)) then
			freezeimg<=not freezeimg;
			debounce_cnt := 10000000; --Wait for 10000000 cycles, or 200 ms before allowing new change
		end if;
		case speed is
		when 1 => shiftfreq <= 7812500;
		when 2 => shiftfreq <= 3906250;
		when 3 => shiftfreq <= 1953125;
		when 4 => shiftfreq <= 781250;	--
		when 5 => shiftfreq <= 390625;	-- 0,5 s/div
		when others =>  shiftfreq <= 2500000;
		end case;
	end if;
	end process;

	char_gen: process (VGA_CLK)
	variable charrow : std_logic_vector(5 downto 0) := "000000";
	variable charcol : std_logic_vector(6 downto 0) := "0000000";
	variable rowcount : std_logic_vector(2 downto 0) := "000";
	variable colcount : std_logic_vector(2 downto 0) := "000";
	variable output : std_logic_vector(0 to 7);
	variable r : std_logic := '0';
	variable g : std_logic := '0';
	variable b : std_logic := '0';
	variable bitout : std_logic := '0';
	variable memval : std_logic_vector(31 downto 0);		--Temporary storage for shifting mem variables
	variable counter : integer := 0;		 -- Counter for updating process variable
	variable timing_corr : integer := 0; -- Correction value for counter. Can update exactly when we want because of memory issues.
	variable memslot : integer := 0;
	variable ramvar : integer := 0;
	variable shiftel : std_logic := '0';
	begin
	if (VGA_CLK'event and VGA_CLK='1') then
		counter := counter + 1;
		if((counter > (shiftfreq-timing_corr)) and (row=0)) then 
			shiftel := '1'; 
			timing_corr := counter - (shiftfreq - timing_corr);
			counter := 0; 
			end if;
		if((shiftel = '1') and (freezeimg = '0')) then
			--Shift elements one to the left
			if(memslot < 639) then 
				if(ramvar = 0) then
					--Ask for next variable
					RAM_addr <= conv_std_logic_vector(memslot+1, 12);
					RAM_we <= "0";
					ramvar := 1;
				elsif(ramvar = 1) then
					--read value
					memval := RAM_dfrom; 
					ramvar := 2;
				elsif(ramvar = 2) then
					--Store in this slot
					RAM_addr <= conv_std_logic_vector(memslot, 12);
					RAM_we <= "1";
					RAM_dto <= memval;
					ramvar := 0;
					memslot := memslot + 1;
				end if;
			elsif(memslot = 639) then 
				--Write next slot
				RAM_addr <= conv_std_logic_vector(memslot, 12);
				RAM_we <= "1";
				RAM_dto(7 downto 0) <= Ref;
				RAM_dto(15 downto 8) <= P_Value(11 downto 4);
				RAM_dto(23 downto 16) <= Out_Value(11 downto 4);
				RAM_dto(31 downto 24) <= P_Value_Mod(11 downto 4);
				memslot := 0; 
				shiftel := '0';
			end if;
		end if;	
		charrow := row(8 downto 3);		--Divide by 8. Coordinates for the characters on screen
		charcol := column(9 downto 3);
		rowcount := row(2 downto 0);		--The row number we are at on the current symbol
		if(column(2 downto 0) = 0) then
			colcount := "111";
		else 
			colcount := column(2 downto 0) - 1;
		end if;
		bitout := '0';
		if(charrow=10) then
			--draw "Referanse"
			case conv_integer(charcol) is
				when 10 =>				char <= "01001011"; -- K
											r := '1';g := '1';b := '1';						
				when 11 =>				char <= "01110000"; -- p
				when 12 =>				char <= "00111101"; -- =
				when 13 =>				char <= "0011" & Kp(7 downto 4); -- Kp value
				when 14 =>				char <= "00101110"; 	-- .
				when 15 =>				char <= "0011" & Kp(3 downto 0); 	-- Kp decimal
				when 20 =>				char <= "01001101"; 						-- M
				when 21 =>				char <= "01100001"; 						-- a
				when 22 =>				char <= "01101110"; 						-- n
				when 23 =>				char <= "00000000"; 						-- 
				when 25 =>				if(mode="00") then 
												char <= "01011000"; 					-- X
											else char <= "00000000";
											end if;
				when 60 | 61 | 62 => char <= "00101101" ; 		-- -
											r := '1';g := '0';b := '0';
				when 63 => 				char <= "00100000";			--	
				when 64 =>				char <= "01010010";	-- R
											r := '1';g := '1';b := '1';	
				when 65 |67 | 72 => 	char <= "01100101";	-- e
				when 66 =>				char <= "01100110";	-- f
				when 68 =>				char <= "01110010";	-- r
				when 69 =>				char <= "01100001";	-- a
				when 70 =>				char <= "01101110";	-- n
				when 71 =>				char <= "01110011";	-- s
				when others =>			char <= "00000000";
			end case;
		elsif(charrow=11) then
			--draw text "Prosess"
			case conv_integer(charcol) is
				when 10 =>				char <= "01010100" ; -- T
											r := '1';g := '1';b := '1';						
				when 11 =>				char <= "01101001" ; -- i
				when 12 =>				char <= "00111101" ; -- =
				when 13 =>				char <= "0011" & Ti(7 downto 4); -- Ti value
				when 14 =>				char <= "00101110"; 	-- .
				when 15 =>				char <= "0011" & Ti(3 downto 0); 	-- Ti decimal
				when 20 =>				char <= "01010000"; 						-- P
				when 21 =>				char <= "01001001"; 						-- I
				when 22 =>				char <= "01000100"; 						-- D
				when 23 =>				char <= "00000000"; 						-- 
				when 25 =>				if(mode="01") then 
												char <= "01011000"; 					-- X
											else char <= "00000000";
											end if;
				when 60 | 61 | 62 => char <= "00101101" ; 		-- -
											r := '0';g := '1';b := '0';
				when 63 => 				char <= "00100000";			-- 
				when 64 =>				char <= "01010000";	-- P
											r := '1';g := '1';b := '1';
				when 65 => 				char <= "01110010";	-- r
				when 66 =>				char <= "01101111";	-- o
				when 67 | 69 | 70 =>	char <= "01110011";	-- s
				when 68 =>				char <= "01100101";	-- e		
				when others =>			char <= "00000000";
			end case;
		elsif(charrow=12) then
			--draw text "Paadrag"
			case conv_integer(charcol) is
				when 10 =>				char <= "01010100" ; -- T
											r := '1';g := '1';b := '1';						
				when 11 =>				char <= "01100100" ; -- d
				when 12 =>				char <= "00111101" ; -- =
				when 13 =>				char <= "0011" & Td; -- Td value
				when 20 =>				char <= "01001101"; 						-- M
				when 21 =>				char <= "01010000"; 						-- P
				when 22 =>				char <= "01000011"; 						-- C
				when 23 =>				char <= "00000000"; 						-- 
				when 25 =>				if(mode="10") then 
												char <= "01011000"; 					-- X
											else char <= "00000000";
											end if;
				when 60 | 61 | 62 => char <= "00101101" ; 		-- -
											r := '0';g := '0';b := '1';
				when 63 => 				char <= "00100000";			-- 
				when 64 =>				char <= "01010000";	-- P
											r := '1';g := '1';b := '1';
				when 65 | 66 | 69 => char <= "01100001";	-- a
				when 67 =>				char <= "01100100";	-- d
				when 68 =>				char <= "01110010";	-- r
				when 70 =>				char <= "01100111";	-- g		
				when others =>			char <= "00000000";
			end case;
		elsif(charrow=13) then
			--draw text "Modell"
			case conv_integer(charcol) is
				when 60 | 61 | 62 => char <= "00101101" ; 		-- -
											r := '1';g := '1';b := '0';
				when 63 => 				char <= "00100000";			-- 
				when 64 =>				char <= "01001101";	-- M
											r := '1';g := '1';b := '1';
				when 65 => 				char <= "01101111";	-- o
				when 66 =>				char <= "01100100";	-- d
				when 67 =>				char <= "01100101";	-- e
				when 68 | 69 =>		char <= "01101100";	-- l		
				when others =>			char <= "00000000";
			end case;
		elsif(charrow=50) then
			--draw "horizontal line"
			if((charcol>=0) and (charcol<80)) then 
				char <= "00101101" ; 		-- -
				if(charcol(1 downto 0) = "00") then
					char <= "00101011" ;
				end if;
				r := '1';g := '1';b := '1';
			else char <= "00000000";
			end if;
		elsif(charcol=40) then
			--draw "vertical line"
			if((charrow>15) and (charrow<50)) then
				char <= "01111100" ;		-- |
				if(charrow(1 downto 0) = "10") then
					char <= "00101011" ;
				end if;
				r := '1';g := '1';b := '1';
			else char <= "00000000";
			end if;
		
		else char <= "00000000";
		end if;
		
		--Read values to plot from block RAM
		if(charrow > 6) then
			RAM_we <= "0";
			memval := RAM_dfrom;			--Read what we asked for previously
			if(column<639) then
				RAM_addr <= "00" & (column+1);	--Ask for the next one
			else
				RAM_addr <= "000000000000";		--Ask the first one
			end if;
		end if;
		
		--"print" plots
		if(memval(7 downto 0)=(403-row)) then
			--draw "reference value"
			bitout := '1';
			r := '1';g := '0';b := '0';
		elsif(memval(15 downto 8)=(403-row)) then
			--draw "process value"
			bitout := '1';
			r := '0';g := '1';b := '0';
		elsif(memval(23 downto 16)=(403-row)) then
			--draw "out value"
			bitout := '1';
			r := '0';g := '0';b := '1';
		elsif(memval(31 downto 24)=(403-row)) then
			--draw "model value"
			bitout := '1';
			r := '1';g := '1';b := '0';
		end if;
		
		--print timescale first row
		if(charrow=53) then
			r := '1';g := '1';b := '1';
			if(charcol=37) then
				case speed is
				when 3 => char <= "00110010"; -- 2
				when 5  => char <= "00110000"; -- 0
				when others => char <= "00000000";
				end case;
			end if;
			if(charcol=38) then
				case speed is
				when 1 => char <= "00110001"; -- 1
				when 3 | 5 => char <= "00101100"; -- ,
				when others => char <= "00000000";
				end case;
			end if;
			if(charcol=39) then
				case speed is
				when 1 => char <= "00110000"; -- 0
				when 2 | 3 | 5 => char <= "00110101"; -- 5
				when 4  => char <= "00110001"; -- 1
				when others => char <= "00000000";
				end case;
			end if;
			if(charcol=40) then
				char <= "01110011" ; -- s
			end if;
			if(charcol=41) then
				char <= "00100000" ;	-- space
			end if;
			if(charcol=42) then
				char <= "00101111" ; -- /
			end if;
			--print "pause" if applicable
			if(freezeimg='1' and charcol > 60) then
				case conv_integer(charcol) is
					when 64 => char <= "01010000"; -- P
					when 65 => char <= "01100001"; -- a
					when 66 => char <= "01110101"; -- u
					when 67 => char <= "01110011"; -- s
					when 68 => char <= "01100101"; -- e
					when others => char <= "00000000";
				end case;
			end if;
		end if;
		
		--print timescale second row
		if(charrow=54) then
			if(charcol=41) then
				char <= "00101111" ; -- /
			end if;
			if(charcol=42) then
				char <= "00100000" ;	-- space
			end if;
			if(charcol=43) then
				char <= "01100100" ;	-- d
			end if;
			if(charcol=44) then
				char <= "01101001" ; 	-- i
			end if;
			if(charcol=45) then
				char <= "01110110" ;	-- v
			end if;
		end if;

		--Print UART input
		if(charrow=56) then
			if(charcol=19) then char <= "00111010"; end if;-- :
			if(charcol=20) then char <= "00000000"; end if;-- space
			if((charcol>20) and (charcol<27)) then  -- data
				char <= UART_DATA(conv_integer(charcol)-21) ;
			end if;
		end if;
		
		case rowcount is
			when "000" => output := vec_0;
			when "001" => output := vec_1;
			when "010" => output := vec_2;
			when "011" => output := vec_3;
			when "100" => output := vec_4;
			when "101" => output := vec_5;
			when "110" => output := vec_6;
			when "111" => output := vec_7;
			when others => output := "00000000";
		end case;
		if(bitout='0') then
			if (output(conv_integer(colcount)) = '1') then
				red <= r;
				green <= g;
				blue <= b;
				output := "00000000";
			else
				red <= '0';
				green <= '0';
				blue <= '0';
			end if;
		end if;
		if(bitout='1') then
				red <= r;
				green <= g;
				blue <= b;
				output := "00000000";
				bitout := '0';
		end if;
	end if;
	end process;
end architecture basic;