--UART--------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use ieee.std_logic_unsigned.all;
use work.global_package.all;

entity uart is
   Port ( UART_CLK 	: in std_logic;
			 UART_RXD 	: in std_logic;
			 UART_TXD 	: out std_logic := '0';
			 UART_DATA 	: inout UART_queue;
			 Kp			: out std_logic_vector(7 downto 0) := "00000111";
			 Ti			: out std_logic_vector(7 downto 0) := "00000101";
			 Td			: out std_logic_vector(3 downto 0) := "0001";
			 mode 		: out std_logic_vector(1 downto 0) := "01";	--default is PID
			 debug		: in std_logic_vector(31 downto 0);
			 Ref 			: out std_logic_vector(7 downto 0) := "00000000"
   );
end uart;

architecture basic of uart is
signal outbyte : std_logic_vector(7 downto 0) := "00000000";
signal send : std_logic := '0';
begin
	read_proc : process (UART_CLK)
	variable inbit 	: integer range 0 to 10 := 0;
	variable inbyte 	: std_logic_vector(7 downto 0) := "00000000";
	variable incoming : std_logic := '1';
	variable charno 	: integer range 0 to 5 := 0;
	variable newcmd 	: std_logic := '0';
	begin
	   if (UART_CLK='1' and UART_CLK'event) then
			if(send='1') then send <= '0'; end if;
			if(incoming = '1') then incoming := UART_RXD; end if;
			if((incoming = '0') and (newcmd='0')) then
				case inbit is
					when 0 => 			inbit := inbit + 1;			
					when 1|2|3|4|5|6|7|8 => 
											inbyte(inbit - 1) := UART_RXD; 	--Data
											inbit := inbit + 1;
					when 9  => 			inbit := inbit + 1;
					when 10 => 			UART_DATA(charno) <= inbyte;
											--outbyte <= inbyte;
											send <= '1';
											if(charno < 5 and inbyte /= "00001000") then charno := charno + 1; end if;	--Get ready for next char
											if(inbyte = "00001101") then  							--Carriage return
												charno := 0; 
												newcmd := '1'; 
											end if; 
											if((inbyte = "00001000") and (charno > 0)) then  	--Backspace
												UART_DATA(charno) <= "00000000";
												UART_DATA(charno-1) <= "00000000";
												charno := charno-1; 
											end if; 	
											inbit := 0;
											incoming := '1';		
				end case;
			end if;
			
			if(newcmd='1') then									-- Try to read command from UART-data
				if(UART_DATA(0)="01001011") then				-- K
					if(UART_DATA(1)="01110000") then			-- p
						if(UART_DATA(2)="00111101") then 	-- =
							Kp(7 downto 4) <= UART_DATA(3)(3 downto 0);	-- Kp value integer
							if(UART_DATA(4)="00101110") then	-- floating point...
								Kp(3 downto 0) <= UART_DATA(5)(3 downto 0);
							else Kp(3 downto 0) <= "0000";
							end if;
						end if;
					end if;
				elsif(UART_DATA(0)="01010100") then			-- T
					if(UART_DATA(1)="01101001") then			-- i
						if(UART_DATA(2)="00111101") then 	-- =
							Ti(7 downto 4) <= UART_DATA(3)(3 downto 0);	-- Ti value integer
							if(UART_DATA(4)="00101110") then	-- floating point...
								Ti(3 downto 0) <= UART_DATA(5)(3 downto 0);
							else Kp(3 downto 0) <= "0000";
							end if;
						end if;
					elsif(UART_DATA(1)="01100100") then		-- d
						if(UART_DATA(2)="00111101") then 	-- =
							Td <= UART_DATA(3)(3 downto 0);	-- Td value
						end if;
					end if;
				elsif(UART_DATA(0)="01101101") then			-- m
					if(UART_DATA(1)="01100001") then			-- a
						if(UART_DATA(2)="01101110") then 	-- n
							mode <= "00";
						end if;
					end if;
					if(UART_DATA(1)="01110000") then			-- p
						if(UART_DATA(2)="01100011") then 	-- c
							mode <= "10";
						end if;
					end if;
				elsif(UART_DATA(0)="01110000") then				-- p
					if(UART_DATA(1)="01101001") then				-- i
						if(UART_DATA(2)="01100100") then 		-- d
							mode <= "01";
						end if;
					end if;
				elsif(UART_DATA(0)="00110000") then
					Ref  <= "00000000";	--0
				elsif(UART_DATA(0)="00110001") then
					Ref  <= "00110010";	--50
				elsif(UART_DATA(0)="00110010") then
					Ref  <= "01100100";	--100
				elsif(UART_DATA(0)="00110011") then
					Ref  <= "10010110";	--150
				elsif(UART_DATA(0)="00110100") then
					Ref  <= "11001000";	--200
				elsif(UART_DATA(0)="00110101") then
					Ref  <= "11111010";	--250
				end if;					
				for charno in 5 downto 0 loop
					UART_DATA(charno) <= "00000000";
				end loop;
				newcmd := '0';
			end if; -- End if newcmd.
		end if;	-- End clk trigger.
	end process read_proc;
	
	write_proc : process (UART_CLK)
	variable outbit : integer range 0 to 10 := 0;
	variable int_send : std_logic := '0';
	variable count : integer range 0 to 33 := 33;
	variable timecnt : integer := 0;
	begin
	   if (UART_CLK='1' and UART_CLK'event) then
			timecnt := timecnt + 1;
			if(timecnt = 9600) then int_send := '1'; timecnt := 0; end if;
			if(int_send = '1') then
				if(count > 1) then outbyte <= "0011000" & debug(count-2);	--Send data
				elsif(count=1) then outbyte <= "00001010";	--Linefeed
				else outbyte <= "00001101";	--Carriage return
				end if;
				case outbit is
					when 0 => 	UART_TXD <= '1';			--Startbit
									outbit := outbit + 1;
					when 1 => 	UART_TXD <= '0';			--Startbit
									outbit := outbit + 1;				
					when 2|3|4|5|6|7|8|9 => 
									UART_TXD <= outbyte(outbit - 2); 	--Data
									outbit := outbit + 1;
					when 10 => 	UART_TXD <= '0';			--Stopbit 1
									outbit := 0;
									if(count=0) then count := 33; int_send := '0';
									else count := count - 1;
									end if;
				end case;
			end if;
		end if;
	end process write_proc;
end architecture basic;