clear all

% Set up model and discretize:
A_C=[-20 -3;3 -0.1];
B_C=[10;0];
C_C=[0 1];
D_C=[0];
Ts=0.05;

[Ad,Bd,Cd,Dd] = C2DM(A_C,B_C,C_C,D_C,Ts,'zoh');

x0=[0;0]; % initial values for the optimization 

% Weightings:
Qx=[0 0;        % State weights
    0 1];
Qx_=[0;1];
Qu=.00;        % Input weight for 0<i<Np-1 Must not be 0!!

% Constraints (implemented as hard here):
umin=[0.0]; umax=[200]; %Make these colums with u1;u2;u3 etc for several inputs
xmin=[-10;0]; xmax=[40;1000];
umin=0.0; umax=200.0;

% Horizon is Np (both for objective function and for constraints);
nx=2;       %Number of states
ny=1;       %Number of measured outputs
nu=1;       %Number of inputs to the system
Np=10;  % Number of prediction samples
Nc=6;
%Nc=2*(nx+nu);   % Number of constraints

% Reference:
x_ref=[0;50];
xref=x_ref;


[ny,nu]=size(Dd);

%Bhat=[zeros(length(Bd),Np)];
Ahat=Ad;
Bhat=[Bd zeros(length(Bd),Np-1)];
for i=2:Np,
    Ahat=[Ahat;Ad^i];
    for j=1:Np,
        if (i-j)>-1,
            Bhat_n(:,j)=Ad^(i-j)*Bd;
        else
            Bhat_n(:,j)=0*Bd;
        end
    end
    Bhat=[Bhat;Bhat_n];
end

% Stack up state and output weights in block-diagonal matrices to
% replace summing in objective function by matrix multiplication in QP:
%{
Qhat=[Qx zeros(nx,nx*(Np-1))];
Quhat=[Qu zeros(nu,nu*(Np-1))];
for i=2:Np,
Qhat_n=[zeros(nx,nx*(i-1)) Qx zeros(nx,nx*(Np-i))];
Quhat_n=[zeros(nu,nu*(i-1)) Qu zeros(nu,nu*(Np-i))];
Qhat=[Qhat;Qhat_n];
Quhat=[Quhat;Quhat_n];
end
%}

% Constraints on states in matrix form: y_min<Hhat*x<y_max
%{
H=Cd;
Hhat=[Cd zeros(ny,nx*(Np-1))];
for i=2:Np,
Hhat_n=[zeros(ny,nx*(i-1)) H zeros(ny,nx*(Np-i))];
Hhat=[Hhat;Hhat_n];
end
%}          


%%%%%%%%%%% SET UP OBJECTIVE FUNCTION AND CALCULATE U %%%%%%%%%%%%%%%%%
%Xdev=Ahat*x0-xref;
Xdev=zeros(nx*Np,1);
for i=1:nx*Np
    for j=1:nx
        Xdev(i)=Xdev(i)+Ahat(i,j)*x0(j);
    end;
end;
for i=1:nx:nx*Np
    for j=1:nx
        Xdev(i+j-1)=Xdev(i+j-1)-xref(j);
    end;
end;
%{
I=eye(Np*nu);
Lu_min=-I;
bu_min(1:Np,1)=-umin;
Lu_max=I;
bu_max(1:Np,1)=umax;
Lx_min=-Bhat;
bx_min=-ymin+(Xdev+xref);
Lx_max=Bhat;
bx_max=ymax-(Xdev+xref);
L1=[Lu_min;Lu_max;Lx_min;Lx_max];
b1=[bu_min;bu_max;bx_min;bx_max]
%}

J=zeros(2*Np*(nu+nx),Np*nu);
%First, insert Lu_min and Lu_max
for i=1:Np*nu
    J(i,i)=-1;
    J(i+Np*nu,i)=1;
end;
%Then, insert Lx_min and Lx_max
for i=1:Np*nx
    for j=1:Np*nu
        J(i+2*Np*nu,j)=-Bhat(i,j);
    	J(i+2*Np*nu+Np*nx,j)=Bhat(i,j);
    end;
end;

g1=zeros(Np*Nc,1);
%First, insert bu_min and bu_max
for i=1:nu:Np
    for j=1:nu
    	g1(i+j-1)=-umin(j);        %Lower bound
        g1(Np*nu+i+j-1)=umax(j);   %Upper bound
    end;
end;
%Then, insert bx_min and bx_max
for i=1:nx:Np*nx
    for j=1:nx
        g1(i+j-1+2*Np*nu)=Xdev(i+j-1)+xref(j)-xmin(j);          %Lower bound
        g1(i+j-1+2*Np*nu+Np*nx)=xmax(j)-Xdev(i+j-1)-xref(j);    %Upper bound
    end;
end;

%Q=Bhat'*Qhat*Bhat+Quhat;
%temp1=Qhat*Bhat;
Q=zeros(nu*Np,nu*Np);
temp1=zeros(nx*Np,nu*Np);
for i=1:nx:nx*Np
    for j=1:nu*Np
        for k=1:nx
            temp1(i+k-1,j)=Qx_(k)*Bhat(i+k-1,j);
        end;
    end;
end;
for i=1:nu*Np
    for j=1:nu*Np
        for k=1:nx*Np
            Q(i,j)=Q(i,j)+Bhat(k,j)*temp1(k,i);
        end;
    end;
end;
for i=1:nu:nu*Np
    for j=1:nu
        Q(i+j-1,i+j-1)=Q(i+j-1,i+j-1)+Qu(j);
    end;
end;
%c=(Xdev'*Qhat*Bhat);
c=zeros(1,nu*Np);
for i=1:nu*Np
    for j=1:nx*Np
        c(i)=c(i)+Xdev(j)*temp1(j,i);
    end; 
end;
%[fasit,noe,iflag,OUTPUT]=quadprog(Q,c,J,g1);

%My MPC algorithm
alpha=0.8;
rho=0.001;
bah1=max([max(abs(Q)) max(abs(c))]);
bah2=max([max(abs(J)) max(abs(g1))]);
Q=Q/bah1;
c=c'/bah1;
J=J/bah2;
g1=g1/bah2;

%solve with quadprog to be able to compare results
[fasit,noe,iflag,OUTPUT,quadlambda]=quadprog(Q,c,J,g1);

%Step 1
z=zeros(Np*nu,1);
l=ones(Np*Nc,1)*1;
t=ones(Np*Nc,1)*1;

%Step 2
my=1;
run=0;
while(my > 0.001)
    %G2=-(L^-1) * T;
    clear G2;
    for i=1:Nc*Np
        G2(i)=-t(i)/l(i);
    end;
    
    %r1=-Q*z-J'*l-c;
    r1=zeros(Np*nu,1);
    for i=1:Np*nu
        r1(i)=-c(i);
        for j=1:Np
            r1(i)=r1(i)-Q(i,j)*z(j);
        end; 
    end;
    for i=1:Np*nu
        for j=1:Np*Nc
            r1(i)=r1(i)-J(j,i)*l(j);
        end;
    end;
    
    %r2=-J*z+g1-rho*my*L^-1*e; 
    r2=zeros(Nc*Np,1);
    for i=1:Nc*Np
        r2(i)=g1(i)-rho*my/l(i);
    end;
    for i=1:Nc*Np
        for j=1:Np
            r2(i)=r2(i)-J(i,j)*z(j);
        end;
    end;

    % ----------- d_z=(Q-J'*(G2^-1)*J)^-1 * (r1-J'*(G2^-1)*r2); ---------
    %First we find J'*(G2^-1)
    temp2=zeros(Np*nu,Nc*Np);
    for i=1:Np*nu
        for j=1:Nc*Np
            temp2(i,j)=J(j,i)/(G2(j));
        end;
    end;

    %Next we find Q-J'*(G2^-1)*J
    temp3=zeros(Np*nu,Np*nu);
    for i=1:Np*nu
        for j=1:Np*nu
            for k=1:Nc*Np
                temp3(i,j)=temp3(i,j)+temp2(i,k)*J(k,j);
            end;
        end;
    end;
    for i=1:Np*nu
        for j=1:Np*nu
            temp3(i,j)=Q(i,j)-temp3(i,j);
        end;
    end;

    %Let's invert it! temp4=temp3^-1
    R=zeros(Np*nu,1);
    temp4=zeros(Np*nu,Np*nu);
    for i=1:Np*nu
        temp4(i,i)=1;
    end;
    for i=1:Np*nu %For all diagonal values, moving down
        n=temp3(i,i);
        for j=1:Np*nu
            temp3(i,j)=temp3(i,j)/n;
            temp4(i,j)=temp4(i,j)/n;
        end;
        for j=1:Np*nu
            R(j)=-(temp3(j,i));
        end;
        R(i)=0;
        for j=i+1:Np*nu    %multiply all rows below the pivot-row
            for k=1:Np*nu
                temp3(j,k)=temp3(j,k)+R(j)*(temp3(i,k));
                temp4(j,k)=temp4(j,k)+R(j)*(temp4(i,k));
            end;
        end; 
    end;
    
    for i=Np*nu:-1:2 %For all diagonal values, moving up
        for j=1:Np*nu
            R(j)=-(temp3(j,i));
        end;
        R(i)=0;
        for j=i-1:-1:1    %multiply all rows above the pivot-row
            for k=1:Np*nu
                temp3(j,k)=temp3(j,k)+R(j)*(temp3(i,k));
                temp4(j,k)=temp4(j,k)+R(j)*(temp4(i,k));
            end;
        end; 
    end;
    
    % We will reuse the top row of temp3 for a new temp row
    %Next we find r1-J'*(G2^-1)*r2
    for i=1:Np*nu
        temp3(1,i)=r1(i);
    end;
    for i=1:Np*nu
        for j=1:Nc*Np
            temp3(1,i)=temp3(1,i)-temp2(i,j)*r2(j);
        end;
    end;
    
    % We will reuse the top two rows of temp2 for calculating the new
    % l and t matrices and d_z
    for i=1:Nc*Np
        temp2(1,i)=0;   %used for l
        temp2(2,i)=0;   %used for t
        temp2(3,i)=0;   %used for d_z
    end;
    
    %Now we can find d_z=(Q-J'*(G2^-1)*J)^-1 * (r1-J'*(G2^-1)*r2);
    %This is temp4 * temp3(1,i). Temp4 is (Np*nu,Np*nu). temp3(1,i) is (Np*nu,1).
    d_z=zeros(Np*nu,1);
    for i=1:Np*nu
        for j=1:Np*nu
            temp2(3,i)=temp2(3,i)+temp4(i,j)*temp3(1,j);
        end;
    end;
    
    % ----------- z=z+alpha*d_z; ---------
    for i=1:Np*nu
        z(i)=z(i)+alpha*temp2(3,i);
    end;
    
    % ----------- d_l=(G2^-1)*r2-(G2^-1)*J*d_z; ---------
    for i=1:Nc*Np
        for j=1:Np*nu
            temp2(1,i)=temp2(1,i)+J(i,j)*temp2(3,j)/G2(i);
        end;
    end;
    for i=1:Nc*Np
        tempvar=l(i) + alpha* ( 1/G2(i)*r2(i)-temp2(1,i) );
        while(tempvar < 0)
            alpha=alpha*0.8;
            tempvar=l(i) + alpha* ( 1/G2(i)*r2(i)-temp2(1,i) );
        end;
        l(i)=tempvar;
        alpha=0.8;
    end;

    % ----------- d_t=-t+g1-J*(z+d_z); ---------
    for i=1:Nc*Np
        for j=1:nu*Np
            temp2(2,i)=temp2(2,i)+J(i,j)*(z(j)+temp2(3,j));
        end;
    end;
    for i=1:Nc*Np
        tempvar=t(i) + alpha*( g1(i)-t(i)-temp2(2,i) );
        while(tempvar < 0)
            alpha=alpha*0.8;
            tempvar=t(i) + alpha*( g1(i)-t(i)-temp2(2,i) );
        end;
        t(i)=tempvar;
        alpha=0.8;
    end;
    
    % ----------- my=t'*l/(Nc*Np); ---------
    my=0;
    for i=1:Nc*Np
        my=my+t(i)*l(i);
    end;
    my=my/(Nc*Np);
    run=run+1;
end

%Same as above, but with most of it on matrix form
%
my=1;
rho=0.05;
e=ones(length(J),1);
z=zeros(Np*nu,1);
l=ones(Np*Nc,1)*1;
t=ones(Np*Nc,1)*1;
run=0;

while(my > 0.0001)
    run=run+1;
    L=diag(l);
    T=diag(t);
    G=-(L^-1) * T;
    r1=-Q*z-J'*l-c;
    r2=-J*z+g1-rho*my*L^-1*e;
    
    J
    d_z=(Q-J'*(G^-1)*J)^-1*(r1-J'*(G^-1)*r2);
    z=z+alpha*d_z;
    
    d_l=(G^-1)*r2-(G^-1)*J*d_z;
    for i=1:Nc*Np
        dummy=l(i)+alpha*d_l(i);
        while(dummy<0)
            alpha=alpha*0.8;
            dummy=l(i)+alpha*d_l(i);
        end;
        l(i)=l(i)+alpha*d_l(i);
        alpha=0.8;
    end; 
    
    d_t=-t+g1-J*(z+d_z);
    for i=1:Nc*Np
        dummy=t(i)+alpha*d_t(i);
        while(dummy<0)
            alpha=alpha*0.8;
            dummy=t(i)+alpha*d_t(i);
        end;
        t(i)=t(i)+alpha*d_t(i);
        alpha=0.8;
    end;   

    my=t'*l/length(J);
end
%
u=z;
%u=fasit; %Fra Matlabs Quadprog
%%%%%%%%%%%%%%% PLOT THE RESULT %%%%%%%%%%%%%%%   

                  
% Save states and input for plotting
for i=1:Np,
    x=Ad*x0+Bd*u(i);
    x0=x;
    X(i,:)=x';
end
% plot time history
t=0:Ts:Np*Ts;t_f=t(length(t));t=[-1 t];
y=[0 0 Cd*X'];
subplot(312),
plot(t,y,[-1 0],[0 0],'r:',[0 0],[0 xref(2)],'r:',[0 t_f],[xref(2) xref(2)],...
      'r:',[-1 t_f],[xmax(2) xmax(2)],'g--')
ylabel('output x2')
axis([-1 t_f min(y)-0.2 max(y)+0.2])

Cd=[1 0];
y=[0 0 Cd*X'];
subplot(311),
plot(t,y,[-1 0],[0 0],'r:',[0 0],[0 xref(1)],'r:',[0 t_f],[xref(1) xref(1)],...
      'r:',[-1 t_f],[xmax(1) xmax(1)],'g--')
ylabel('output x1')
axis([-1 t_f min(y)-0.2 max(y)+5])

subplot(313)
U=[0 u' u(length(u))];
stairs(t,U)
axis([-1 t_f umin(1)-0.1 umax(1)+0.1])
ylabel('input u')
xlabel('time [s]')


tsim=[0:Ts:Ts*(Np-1)]';
simin=[tsim u];